/* 
**  CXSC is a C++ library for eXtended Scientific Computing (V 2.2)
**
**  Copyright (C) 1990-2000 Institut fuer Angewandte Mathematik,
**                          Universitaet Karlsruhe, Germany
**            (C) 2000-2006 Wiss. Rechnen/Softwaretechnologie
**                          Universitaet Wuppertal, Germany   
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Library General Public
**  License as published by the Free Software Foundation; either
**  version 2 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Library General Public License for more details.
**
**  You should have received a copy of the GNU Library General Public
**  License along with this library; if not, write to the Free
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* CVS $Id: lx_imath.cpp,v 1.18 2006/12/06 13:11:25 cxsc Exp $ */

#include "lx_imath.hpp"
#include <cmath> 


namespace cxsc {

lx_interval sqrt(const lx_interval &a) throw(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF)
{
    int stagsave = stagprec,
	stagmax = 30; // l_imath.cpp: sqrt() uses stagmax = 30;
    if (stagprec>stagmax) stagprec = stagmax;

    lx_interval res;
    l_interval la(li_part(a));
    int na(expo(a)),exa(expo_gr(la)),d;

    if (exa<-100000) res = 0.0; // la = 0;
    else // la <> 0;
    {
	d = 1024 - exa; // d >= 0;
	na = sub_int(na,d);
	if (na%2 != 0)
	{ na = add_int(na,1); d--; }
	Times2pown(la,d);
	na = na/2;
	la = sqrt(la);
	res = lx_interval(na,la);
    }
    stagprec = stagsave;
    res = adjust(res);
    return res;
} // sqrt()

lx_interval sqr(const lx_interval &a) throw()
{
    int stagsave = stagprec,
	stagmax = 39;
    if (stagprec>stagmax) stagprec = stagmax;

    lx_interval res;
    l_interval la(li_part(a));
    int na(expo(a)),exa(expo_gr(la)),d;

    if (exa<-100000) res = 0.0; // la = 0;
    else // la <> 0;
    {
	d = 511 - exa;
	Times2pown(la,d);
	d = sub_int(na,d);
	na = add_int(d,d);
	la = sqr(la);
	res = lx_interval(na,la);
    }
    stagprec = stagsave;
    res = adjust(res);
    return res;
} 

lx_interval Lnp1(const lx_interval &x) throw(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF)
// Calculating an inclusion of ln(1+x);
{
    lx_interval res(0),z2,zeta,Ri;
    l_interval xli;
    int m,N,ex,p,expox;

    p = StagPrec(x);
    xli = li_part(x);
    ex = expo_gr(xli);
    if (ex>-100000) // x <> 0
    {
	expox = expo(x);
	if (expox<0)
	    if (expox+2147482626 < -ex) // zeta calculation 
                                        // with int-Overflow!
		cxscthrow(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF(
		     "lx_interval Lnp1(const lx_interval &)"));
	if (expox < 3-27*p-ex) N = 0;
	else
	{
	    m = ex + expox;  // m = ex + expo(x);
	    N = (53*p-4)/(2*(1-m)); 
	}
        // N: Polynomial degree, 0<=N<=42;

	zeta = x / (2+x); // int-Overflow impossible!
	Ri = lx_interval( Sup(abs(zeta)) );
	if (N==0)
	{
	    res = lx_interval(2.0);
	    if (expox >= -1073741312-ex)
		Ri = sqr(Ri); // Without int-Overflow!
	}
	else // N >= 1:
	{
	    z2 = sqr(zeta); // z2 = zeta^2
	    // Evaluating the polynomial:
	    res = lx_interval(2.0) / (2*N+1);  // res = a_(N)
	    for (int i=N-1; i>=0; --i)
		res = res*z2 + lx_interval(2)/(2*i+1);
	    // Calculating the absolute approximation error: 
	    Ri = sqr(Ri);
	    Ri = power(Ri,N+1)/(N+1);
	}
	// Implementing the approximation error:
	res += lx_interval(lx_real(0),Sup(Ri));
	res *= zeta;
    } // x <> 0;

    return res;
} // Lnp1(...)

lx_interval ln(const lx_interval &x) throw(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF) 
{
    int stagsave = stagprec,
	stagmax = 39;
    if (stagprec>stagmax) stagprec = stagmax;

    lx_interval T(0.0);
    l_interval lx(li_part(x));
    interval z;
    real eps;
    int exa(expo_gr(lx)),nx(expo(x)),n,k(0);

    if (Inf(lx)<=0) cxscthrow(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF(
				  "lx_interval ln(const lx_interval &)"));
    // Now we have: Inf(x)>0 --> exa <> -2147483648:
    n = add_int(exa,nx);
    if (n==0 || n==1)
    {
	T = x - 1.0;
	lx = T;
	if (Sup(abs(lx))<1e-7)
	{
	    T = Lnp1(T);
	    goto fertig;
	}
    }
    T = x;
    T = adjust(x);
    times2pown(T,-n); // T: 1. reduced argument
    lx = T;
    z = lx;
    eps = Sup(1-z);
    if (eps>1e-12)
    {
	eps = (ln(eps) + 6*ln_N[8])/ln_N[0]; 
	k = (int) _double(eps);
	if (k<=0) k = 0;
    }
    for (int j=1; j<=k; j++) 
	T = sqrt(T); // T: 2. reduced argument,
	             // T = [2^k]sqrt(T);
    T = Lnp1(T-1.0);
    if (k>0) times2pown(T,k);
    T = n*Ln2_lx_interval() + T;
 fertig:
    stagprec = stagsave;
    T = adjust(T);

    return T;
} // ln(...)


lx_interval lnp1(const lx_interval &x) throw()
{
    const real r = 1e-7;
    int stagsave = stagprec,
	stagmax = 39;
    if (stagprec>stagmax) stagprec = stagmax;
    lx_interval res;

    res = (Inf(x)>-r && Sup(x)<r)? Lnp1(x) : ln(1.0+x); 

    stagprec = stagsave;
    res = adjust(res);

    return res;
}

lx_interval power(const lx_interval &x, int n)
{
    int stagsave = stagprec,
	stagmax = 38;
    real one(1.0);              
    bool neg = false;
    long int zhi = 2;
    stagprec = StagPrec(li_part(x));
    if (stagprec>stagmax) stagprec = stagmax;

    lx_interval  y, neu;

    if (x == one) y = x;
    else 
	if (n == 0) y = one;
	else 
	{
	    if (stagprec < stagmax) 
		stagprec++;
	    else stagprec = stagmax;

	    if (n == 1)
		y = x;
	    else 
		if (n == 2) y = sqr(x);
		else 
		{
		    if (n < 0) 
		    {
			neg = true;
			n = -n;
		    }
		    // Initialisierung
		    if (n%2) y = x;
		    else y = one;  // Praezision wird bei 1 Mult. 
                                   // auf aktuellen Wert gesetzt;
		    // Berechnugn durch binaere Darstellung der n
		    neu = sqr(x);   // neu = x*x;
		    do {
			if ((n/zhi)%2)  y *= neu;
			zhi = add_int(zhi,zhi);  // zhi += zhi;
			if (zhi <= n)  // letzte Mult. entfaellt --> schneller
			    neu *= neu;
		    } while (zhi <= n);
		    
		    if (neg) y = 1.0/(y);
		}
	}
    stagprec = stagsave;
    y = adjust(y);
    return y;
}

// -------------------- Exponential function -----------------------------

int Poly_N_Exp(const lx_interval &x)
// Calculation of the polynomial degee N:
// Only for internal use in exp-function !
{
    lx_interval absx(abs(x));
    int N(0),
	m;
    real S(-53*StagPrec(x)*ln_N[0]),
	c,
	D; // S: -53*(precision of x)*ln(2);

    m = expo_gr(li_part(absx)); 
    if (m>-1000000)
    {   // x != 0:
	m = add_int(m,expo(absx));
	c = m*ln_N[0];
	D = c;  // D(0) = m*ln(2);
	while (D > S) {
	    N++;
	    D = D + c - ln_N[N-1];
	}
    }

    return N;
}


lx_interval exp(const lx_interval &x) throw(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF)
{
    int stagsave = stagprec,
	stagmax = 40,
	ex,m,n,N;
    if (stagprec>stagmax) stagprec = stagmax;
    if (stagprec<3) stagprec = 3;

    lx_interval res(1.0),absx,T;
    lx_real S;

    absx = abs(x);
    if (Sup(absx)>1488521882)
	cxscthrow(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF(
		      "lx_interval exp(const lx_interval &x)"));

    ex = expo_gr(li_part(absx));
    if (ex>-1000000)
    { // x != 0
	m = add_int(expo(absx),ex); // absx '=' 2^m
	n = (int) _double( (9*ln_N[8]+m*ln_N[0])/ln_N[0]);
	T = x;
	T = adjust(T);
	if (n>0)
	    times2pown(T,-n); // T '=' 10^-9
        // T: reduced argument
	N =  Poly_N_Exp(T); // N: Polynomial degree
        // Calculation of the N+1 polynomial coefficients
        // Koff[0], ... , Koff[N]:
	lx_interval Koff[N+1];
	Koff[0] = 1.0;
	for (int k=1; k<=N; k++)
	    Koff[k] = Koff[k-1]/k;
        // Horner-evaluation of the polynomial P_N(T):
	res = Koff[N];
	for (int k=N-1; k>=0; k--)
	    res = res*T + Koff[k];
        // Calculation of the absolute approximation error:
	Koff[0] = 1.0;
	for (int k=1; k<=N+1; k++)
	    Koff[0] = Koff[0]*k;  // Koff[0] = (N+1)!
	T = lx_interval( Sup(abs(T)) );
	T = (power(T,N+1)/Koff[0])/(1-T);
        // T: Inclusion of the absolute approximation error
	S = Sup(T);
	res = res + lx_interval(-S,S); // We now considered 
        // the absolute approximation error of P_N(T).
        // the following loop is only used if n = 1,2,3,...
	for (int k=1; k<=n; k++)
	    res = sqr(res);
    }
    stagprec = stagsave;
    res = adjust(res);

    return res;
} // exp(...)   

lx_interval exp2(const lx_interval &x) throw()
{
    int stagsave = stagprec,
	stagmax = 40;
    if (stagprec>stagmax) stagprec = stagmax;
    if (stagprec<3) stagprec = 3;

    lx_interval res;
    res = exp( x*Ln2_lx_interval() );

    stagprec = stagsave;
    res = adjust(res);

    return res;
} // exp2(...)  

lx_interval exp10(const lx_interval &x) throw()
{
    int stagsave = stagprec,
	stagmax = 40;
    if (stagprec>stagmax) stagprec = stagmax;
    if (stagprec<3) stagprec = 3;

    lx_interval res;
    res = exp( x*Ln10_lx_interval() );

    stagprec = stagsave;
    res = adjust(res);

    return res;
} // exp10(...)   


lx_interval pow(const lx_interval &x, const lx_interval &e) 
                                   throw(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF)
{
    int stagsave = stagprec,
	stagmax = 40,
	intexp;
    lx_interval y;
    lx_real supabse = Sup(abs(e));
    l_interval de;
    l_real lr;
    real r;
    bool fertig(false);

    if (point_intv(e))
	if (supabse < 2147483647)
	{
	    de = e;
	    lr = Sup(de);
	    intexp = int(_double(real(lr)));
	    if (real(intexp) == Sup(e))
	    {
		y = power(x,intexp);
		fertig = true;
	    }
	}

    if (!fertig)
    {
	if (sm_zero(Inf(x))) cxscthrow(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF("lx_interval pow(const lx_interval&, const lx_interval&)"));
	if (point_intv(x) && Sup(x) == 1.0)
	    y = x;
	else 
	    if (point_intv(e) && Sup(e) == 1.0)
		y = x;
	    else
		if (point_intv(e) && Sup(e) == 0.0)
		    y = 1.0;
		else
		{
		    if (stagprec < stagmax) stagprec++;
		    else stagprec = stagmax;
		    y = exp(e*ln(x));
		    stagprec = stagsave;
		    y = adjust(y);
		}
    }

    return y;
} // pow(...)

lx_interval xp1_pow_y(const lx_interval &x, const lx_interval &y) 
                                   throw(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF)
{
    int stagsave = stagprec,
	stagmax = 40;
    if (stagprec>stagmax) stagprec = stagmax;
    lx_interval res, su(1.0+x);

    if (sm_zero(Inf(1+x))) cxscthrow(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF("lx_interval xp1_pow_e(const lx_interval&, const lx_interval&)"));
    if (point_intv(su) && Sup(su) == 1.0)
	res = 1.0;
    else 
	if (point_intv(y) && Sup(y) == 1.0)
	    res = su;
	else
	    if (point_intv(y) && Sup(y) == 0.0)
		res = 1.0;
	    else
	    {
		if (stagprec < stagmax) stagprec++;
		else stagprec = stagmax;
		res = exp(y*lnp1(x));
		stagprec = stagsave;
		res = adjust(res);
	    }

    return res;
} // xp1_pow_y(...)

int Poly_N_Expm1(const lx_interval &x) 
// Calculation of the polynomial degee N:
// Only for internal use in Expm1(...) 
{
    lx_interval absx(abs(x));
    int N(0),
	m;
    real S(-53*StagPrec(x)*ln_N[0]),
	c,
	D; // S: -53*(Praezision von x)*ln(2);

    m = expo_gr(li_part(absx)); // Neu
    if (m>-1000000)
    {
	m = add_int(m,expo(absx));
	c = m*ln_N[0];
	D = (m-1)*ln_N[0];  // D = (m-1)*ln(2);
	while (D > S) {
	    N++;
	    D = D + c - ln_N[N];
	}
    }

    return N;
}

lx_interval Expm1(const lx_interval &x)
//  e^x - 1;
//  Only for internal use! 
{
    lx_interval res(0),Fak,D;
    lx_real S;
    int N;

    N = Poly_N_Expm1(x);
    // Berechnung der benoetigten (N+1) Taylor-Koeffizienten
    lx_interval Koff[N+1];
    Koff[0] = 1.0;
    for (int k=1; k<=N; k++)
	Koff[k] = Koff[k-1]/(k+1);
    // Berechnung des Taylor-Polynoms P_N(x) (Hornerschema:)
    res = Koff[N];
    for (int k=N-1; k>=0; k--)
	res = res*x + Koff[k];
    // Berechnung des abs. Approximationsfehlers:
    Fak = 1.0;
    for (int k=1; k<=N+2; k++)
	Fak = Fak * k;  // Fak = (N+2)!
    D = lx_interval( Sup(abs(x)) );
    D = (power(D,N+1)/Fak)/(1-D); // D: Einschliessung des
                                  // abs. Approximaitonsfehlers;
    S = Sup(D);
    res = res + lx_interval(-S,S); // Beruecksichtg. des Approx-Fehlers;
    res = res * x;

    return res;
}


lx_interval expm1(const lx_interval &x) throw()
// e^x - 1;
{
    int stagsave = stagprec,
	stagmax = 40;
    if (stagprec>stagmax) stagprec = stagmax;
    lx_interval res;
    l_interval t(li_part(x));

    if (0<=t) 
    {
	t = x; // liefert nur in Extremfaellen Overflow!
	if (Sup(abs(t))<MinReal)
	    res = Expm1(x);
	else 
	    res = expm1(t);
    }
    else 
    {
	if (Sup(abs(x))<1e-7)
	    res = Expm1(x);
	else {
	    if (stagprec < stagmax) stagprec++;
	    else stagprec = stagmax;
	    res = exp(x)-1;
	}
    }
    stagprec = stagsave;
    res = adjust(res);

    return res;
} // expm1(...)

// -------------------------- sin(...) --------------------------------------

real pot3_n[20] = {3.0, 9.0, 27.0, 81.0, 243.0, 729.0, 2187.0, 6561.0,
                   19683.0, 59049.0, 177147.0, 531441.0, 1594323.0, 4782969.0,
                   14348907.0, 43046721.0, 129140163.0, 387420489.0,
                   1162261467.0, 3486784401.0 }; 
// pot3_n[k] = 3^(k+1); k = 0,1,2, ... ,19;
// pot3_n[0] = 3.0; ...  pot3_n[19] = 3^20 = 3486784401.0; 

lx_interval Sin_Rek(const lx_interval &x, int n)
// Only for internal use in sin-function.
{
    lx_interval u,v;
    u = x;
    for (int k=1; k<=n; k++)
    { // Rekursion:  u = u*(3-4*u^2);
	v = sqr(u);
	times2pown(v,2); // v = 4*sqr(u);
	u = u*(3-v);
    }
    return u;
}

int Poly_N_Sin(const lx_interval &x)
// Calculation of the polynomial degee N:
// Only for internal use in sin-function.
{
    lx_interval absx(abs(x));
    int N(0),
	k;
    real S(-53*StagPrec(x)*ln_N[0]), // S: -53*(pecision of x)*ln(2);
	c,
	D; 

    k = expo_gr(li_part(absx));
    if (k>-1000000) 
    {   // x != 0
	k = add_int(k,expo(absx));
	c = (2*k)*ln_N[0];
	D = c - ln_N[4];  // D = D(0) = 2k*ln(2) - ln(3!);
	while (D > S) {
	    N++;
	    D = D + c - ln_N[2*N] - ln_N[2*N+1]; // Recursion formula
	}
    }  // N=0 if x==0;
    return N;
}


l_real floor(const l_real &x)
// Rounding to the next integer number z, with z <= x;
{
    l_real y(x),z(0);
    int p(StagPrec(y)),k;

    y = y + 0; // Sorting the components.

    if (expo(y[1])>-1000000)
    // x != 0
    {
	k=1;
	while(expo(y[k])>=53 && k<=p)
	{
	    z += y[k]; // Addition of the integer parts
	    k++;
	}
	if (k<=p)
	    z += std::floor(_double(y[k]));  // Addition of the last
    }                                        // floor part of y[k].

    return z;
} // floor

lx_interval sin(const lx_interval &x)
// Inclusion of sin(x)
{
   int stagsave = stagprec,
       stagmax = 39,
       n,N,ex,exl;
    const real ln2 = 0.6931471806,
	ln10 = 2.302585093,
	ln3r = 0.9102392264; // 1/ln(3)
    if (stagprec > stagmax) 
         stagprec = stagmax;
    lx_interval res(0.0),T,Pi2;
    lx_real S,Si;
    l_real m;
    l_interval xl;
    interval z;

    if (Sup(abs(x))>MaxReal) 
	return lx_interval(0,l_interval(-1,1));

    xl = x; // Falls |x| zu gross --> Fehlermeldung!
    ex = expo(x);
    exl = expo_gr(li_part(x));
    if (exl>-1000000)
    {  // x != 0
	if (ex<-1080-exl) // sin(x) approx x
	    if (0<=x)
		res = lx_interval(Inf(x),Sup(x));
	    else 
		if (Inf(x)>0)
		{
		    Pi2 = lx_interval(Inf(x),Inf(x));
		    T = Pi2 * One_m_lx_interval();
		    res = lx_interval(Inf(T),Sup(x));
		}
		else 
		{
		    Pi2 = lx_interval(Sup(x),Sup(x));
		    T = Pi2 * One_m_lx_interval();
		    res = lx_interval(Inf(x),Sup(T));
		}
	else 
	{
	    // 1. Argument reduction:
	    // Calculation of m:
	    Pi2 = Pi_lx_interval();
	    times2pown(Pi2,1);  // Pi2 = 2*Pi
	    m = floor( Inf(xl)/Inf(Pi2_l_interval()) ); 
	    T = x - m*Pi2;  // T: reduced argument
	    xl = T;  z = xl;

	    if (diam(z)>Pi2_real)
		res = lx_interval(real(-1.0),real(1.0));
	    else // diam(z) <= 2*Pi:
	    {
		n = expo( Sup(abs(z)) );
		n = (int) _double((n*ln2 + 8*ln10)*ln3r);

		// if (n<=0) then |z|<=10^(-8), 
		// i.e. a second argument reduction is not necessary!
		// A second argument reduction exist  <==>  n>0.

		if (n>0)
		    T = T / pot3_n[n-1]; // T = T / 3^n;
		// T is the reduced argument;
		N = Poly_N_Sin(T); // N: Polynomial degree;

		// Calculation of the N+1 polynomial coefficients
		// Koff[0], ... , Koff[N]: 
		lx_interval Koff[N+1];
		Koff[0] = 1.0;
		for (int k=1; k<=N; k++)
		{
		    m = 2*k;
		    Koff[k] = -Koff[k-1]/(m*(m+1));
		}

		// Horner-evaluation of the polynomial P_N(T^2):
		res = Koff[N];  Pi2 = sqr(T);
		for (int k=N-1; k>=0; k--)
		    res = res*Pi2 + Koff[k];

		// Calculation of the absolute approximation error: 
		Koff[0] = 1.0;
		for (int k=1; k<=N+1; k++)
		{
		    m = 2*k;
		    Koff[0] = Koff[0]*m*(m+1);
		}  // Koff[0] = (2N+3)!
		Pi2 = lx_interval( Sup(abs(T)) );
		Pi2 = sqr(Pi2);
		Pi2 = power(Pi2,N+1)/Koff[0];
		// Pi2: Inclusion of the absolute approximation error
		S = Sup(Pi2);
		res = res + lx_interval(-S,S); // We now considered 
		// the absolute approximation error of P_N(T^2).
		res = res*T; // res: Inclusion of sin(T/3^n)
		// Now calculating an inclusion of sin(T) using
		// the recursion:
		res = Sin_Rek(res,n);
	    }
	}
    }
    stagprec = stagsave; // Restore the old stagprec value
    res = adjust(res);   

    S = Sup(res);  Si = Inf(res);
    if (S>1.0)   res = lx_interval(Si,lx_real(1.0));
    if (Si<-1.0) res = lx_interval(lx_real(-1.0),S);
    // Now   -1 <= res <= +1   is realized
    return res;
} // sin(...)


lx_interval sin_n(const lx_interval &x, int n)
// Inclusion of sin(n*Pi + x)
{
   int stagsave = stagprec,
       stagmax = 39;
    if (stagprec > stagmax) 
         stagprec = stagmax;
    lx_interval res;

    res = sin(x);
    if (n%2 != 0) res = -res;

    stagprec = stagsave; // Restore the old stagprec value
    res = adjust(res);   

    return res;
} // sin_n(...)

lx_interval cos(const lx_interval &x)
// Inclusion of cos(x)
{
   int stagsave = stagprec,
       stagmax = 39;
    if (stagprec > stagmax) 
         stagprec = stagmax;
    lx_interval res;
    lx_real S,Si;

    if (li_part(x)==0) res = 1.0;
    else
    {
	res = Pi_lx_interval();
	times2pown(res,-1); // res = Pi/2;
	res = x + res;
	res = sin(res);
    }

    stagprec = stagsave; // Restore the old stagprec value
    res = adjust(res);   

    S = Sup(res);  Si = Inf(res);
    if (S>1.0)   res = lx_interval(Si,lx_real(1.0));
    if (Si<-1.0) res = lx_interval(lx_real(-1.0),S);
    // Now   -1 <= res <= +1   is realized

    return res;
} // cos(...)

lx_interval cos_n(const lx_interval &x, int n)
// Inclusion of cos((n+0.5)*Pi + x)
{
   int stagsave = stagprec,
       stagmax = 39;
    if (stagprec > stagmax) 
         stagprec = stagmax;
    lx_interval res;

    res = sin(x);
    if (n%2 == 0) res = -res;

    stagprec = stagsave; // Restore the old stagprec value
    res = adjust(res);   

    return res;
} // cos_n(...)

lx_interval tan(const lx_interval & x) throw(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF) 
{
    lx_interval c,y;

    if (li_part(x) == 0)
	y = lx_interval(0,l_interval(0.0));
    else 
    {
	c = cos(x);
	if (0.0 <= c) 
	{
	   cxscthrow(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF("lx_interval tan(const lx_interval &x)"));
	}
	y = sin(x)/c;
    }
    return y;
} // tan()

lx_interval cot(const lx_interval & x) throw(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF) 
{
    lx_interval s,y;

    s = sin(x);
    if (0.0 <= s) 
    {
	cxscthrow(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF("lx_interval cot(const lx_interval &x)"));
    }
    y = cos(x)/s;

    return y;
} // cot()

lx_interval sqrt1px2(const lx_interval &x) throw() 
{   // Inclusion of sqrt(1+x^2)
    int ext,ex,c(3210);
    lx_interval y(abs(x));

    ext = expo_gr(li_part(y));
    ex = expo(y);

    if (ext>-1000000 && ex>c)
        y = y + l_interval(0,1);
    else 
	if (ext<-1000000) y = lx_interval(0,l_interval(1));
	else // x != 0;
	    y = (ex<-c)? 1.0 + lx_interval(lx_real(0),Sup(y)) : sqrt(1+sqr(y));

    return y;
} // sqrt1px2()

int Poly_N_Atan(const lx_interval &x)
// Calculation of the polynomial degee N:
// Only for the internal use in the atan-function.
{
    lx_interval absx(abs(x));
    int N(0),
	ex;
    real S(-53*StagPrec(x)*ln_N[0]), // S: -53*(pecision of x)*ln(2);
	c,
	D; 
    ex = expo_gr(li_part(absx));
    if (ex>-1000000) 
    {   // x != 0
	ex = ex + expo(absx);
	c = ex*ln_N[0];
	D = 2*ex*ln_N[0] - ln_N[1];  // D = D(0) = 2ex*ln(2)-ln(3);
	while (D > S) {
	    N++;
	    D = (2*N+2)*c - ln_N[2*N+1]; 
	}
    }  // N=0 if x==0;
    return N;
}

lx_interval atan(const lx_interval &x) throw() 
// Inclusion of atan(x)
{ 
    const int c(4); 
    int stagsave = stagprec,
	stagmax = 39,
	ex(expo(x)), exl, m(0), N;
    if (stagprec>stagmax) stagprec = stagmax;

    lx_interval res(0.0),T,T2;
    l_interval xl(li_part(x));
    bool zero_inx(0<=xl), neg;
    lx_real Xl;

    exl = expo_gr(xl);
    if (exl>-100000)
    { // x != 0
	res = x;   neg = ex>0;
	if ( !zero_inx && neg && abs(Inf(res))>lx_real(2092,8.567562) )
	{  // 0 not enclosed by x:
	    neg = Sup(xl)<0;
	    if (neg) res = -res;
	    res = Pi_lx_interval();
	    times2pown(res,-1); // Pi/2 enclosed by res;
	    if (neg) res = -res;
	}
	else
	    if (ex<-1080-exl) // atan(x) approx x
		if (0<=x)
		    res = lx_interval(Inf(x),Sup(x));
		else
		    if (Inf(x)>0)
		    {
			T2 = lx_interval(Inf(x),Inf(x));
			T = T2 * One_m_lx_interval();
			res = lx_interval(Inf(T),Sup(x));
		    }
		    else
		    {
			T2 = lx_interval(Sup(x),Sup(x));
			T = T2 * One_m_lx_interval();
			res = lx_interval(Inf(x),Sup(T));
		    }
	    else
	    {   // Now with argument reduction (Rothmaier)
		ex = ex + exl;
		if (ex>1) 
		    m = (int) _double( (ln_N[0]+c*ln_N[8])/ln_N[0] );
		else
		    if (ex>-10000)
			m = (int) _double((c*ln_N[8] + ex*ln_N[0])/ln_N[0]);
		T = x;
		// Now with m >= 0 transformations:
		for (int k=1; k<=m; k++) 
		    T = T /(1 + sqrt1px2(T)); // T: reduced argument
		// Now calculating the polynomial degree:
		N = Poly_N_Atan(T);
		// Calculation of the N+1 polynomial coefficients
		// Koff[0], ... , Koff[N]:
		lx_interval Koff[N+1];
		Koff[0] = 1.0;
		for (int k=1; k<=N; k++)
		{
		    Koff[k] = lx_interval(0,l_interval(1))/ (2*k+1);
		    if (k%2 != 0) Koff[k] = -Koff[k];
		}
		// Horner-evaluation of the polynomial P_N(T2):
		res = Koff[N]; 
		T2 = sqr(T);
		for (int k=N-1; k>=0; k--)
		    res = res*T2 + Koff[k]; // res = P_N(T2)
		// Calculation of the absolute approximation error:
		T2 = lx_interval( Sup(abs(T)) );
		T2 = sqr(T2);
		T2 = power(T2,N+1)/(2*N+3);
		// T2: Inclusion of the absolute approximation error
		// Implementing the approximation error:
		Xl = Sup(T2);
		res += lx_interval(-Xl,Xl);
		res = T*res;
		if (m>0) times2pown(res,m);
	    }
    }

    stagprec = stagsave;
    res = adjust(res);

    return res;
} // atan()

// ---------------------- sqrt(1-x^2) -----------------------------------

lx_interval sqrt1mx2(const lx_interval &x) throw() 
// Inclusion of sqrt(1-x^2)
// Blomquist, 18.07.07;
{ 
    int stagsave = stagprec,
    stagmax = 39,
	ex(expo(x)), exl;
    if (stagprec>stagmax) stagprec = stagmax;
    if (stagprec==1) stagprec++;

    lx_interval res(1);
    l_interval li(li_part(x));

    exl = expo_gr(li);
    if (exl>-100000)
    { // x != 0
	if (ex<=-2098-exl)
	{   // Construction of the inclusion 
            // interval [1 - 2^-2097,1], only possible if p>=2; 
	    res = lx_interval(-1023,l_interval(comp(0.5,1024)));
	    li = li_part(res);
	    li += 0; // To get the actual precision;
	    li[StagPrec(li)] = -minreal; // minreal = 2^-1074;
	    res = lx_interval(-1023,li); 
            // res = 2^-1023*[2^1023 - 2^-1074,2^1023] = [1-2^-2097,1];
	}
	else 
	    res = (ex>=-exl)? sqrt( (1-x)*(1+x) ) : sqrt( 1-sqr(x) );
    }

    stagprec = stagsave;
    res = adjust(res);

    return res;
} // sqrt(1-x^2)

// ------------------- sqrt(x^2-1) ------------------------------------

lx_interval sqrtx2m1(const lx_interval &x) 
                             throw(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF) 
// Inclusion of sqrt(x^2-1)
// Blomquist, 01.11.07;
{ 
    int stagsave = stagprec,
    stagmax = 39,
	ex(expo(x));
    if (stagprec>stagmax) stagprec = stagmax;

    lx_interval y(abs(x)),res;
    l_interval li(li_part(y));
    lx_real Infy(Inf(y));

    if (Infy < 1)
    {
	cxscthrow(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF(
                            "lx_interval sqrtx2m1(const lx_interval &x)"));
    }
    // Now: Inf(y) >= 1;
    if (ex > 1604-expo_gr(li))
	res = y + lx_interval(lx_real(-1),lx_real(0));
    else 
    {
	res = sqrt( (y-1)*(y+1) );
	if (Inf(res)<=0 && Infy>1)
	{
	    y = lx_interval(Infy);
	    y = y + 1;
	    times2pown(y,-2097);
	    y = sqrt(y);
	    res = lx_interval( Inf(y), Sup(res) );
	}
    }

    stagprec = stagsave;
    res = adjust(res);

    return res;
} // sqrt(x^2-1)

/*
lx_interval sqrtx2m1(const lx_interval &x) 
                             throw(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF) 
// Inclusion of sqrt(x^2-1)
// Blomquist, 19.07.07;
{ 
    int stagsave = stagprec,
    stagmax = 39,
	ex(expo(x));
    if (stagprec>stagmax) stagprec = stagmax;

    lx_interval y(abs(x));
    l_interval li(li_part(y));

    if (Inf(y) < 1)
    {
	cxscthrow(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF(
                            "lx_interval sqrtx2m1(const lx_interval &x)"));
    }
    // Now: Inf(y) >= 1;
    if (ex > 1604-expo_gr(li))
	y = y + lx_interval(lx_real(-1),lx_real(0));
    else 
	y = sqrt( sqr(y)-1 );

    stagprec = stagsave;
    y = adjust(y);

    return y;
} // sqrt(x^2-1)
*/
// ------------------- asin(x) ----------------------------------------

lx_interval asin(const lx_interval& x) throw(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF)
{
    int stagsave = stagprec,
	stagmax = 39;
    if (stagprec > stagmax) 
	stagprec = stagmax;

    lx_interval y,pihalbe,h;
    bool neg;

    if (Inf(x)<-1 || Sup(x)>1) 
	cxscthrow(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF
		  ("lx_interval asin(const lx_interval& x)"));

    l_interval xl;
    interval z;
    xl = x;
    z = xl;

    real supabsz = Sup(abs(z)),
	infz = Inf(z),
	supz = Sup(z);

    pihalbe = Pi_lx_interval();
    times2pown(pihalbe,-1); // pihalbe = pi/2;

    if (infz == supz && supz == 0)
	y = 0;
    else 
	if (infz == supz && supabsz == 1)
	{
	    if (supz == 1.0)
		y = pihalbe;
	    else y = -pihalbe;
	}
	else
	{
	    if (0<=z)
		y = atan(x/sqrt1mx2(x));
	    else 
	    {
		y = x;
		neg = Sup(z)<0;
		if (neg) y = -y;
		if (supabsz < 0.75)
		    y = atan(y/sqrt1mx2(y));
		else 
		    y = pihalbe - atan(sqrt1mx2(y)/y);
		if (neg) y = -y;
	    }
	}
    stagprec = stagsave; // Restore the old stagprec value
    y = adjust(y); 

    return y;
} // asin(x)


// ------------------- acos(x) ----------------------------------------

lx_interval acos(const lx_interval& x) throw(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF)
{
// Blomquist, 31.07.2007;
    int stagsave = stagprec,
	stagmax = 39;
    if (stagprec > stagmax) 
	stagprec = stagmax;

    lx_interval y,pihalbe,pi;
    bool neg;

    if (Inf(x)<-1 || Sup(x)>1) 
	cxscthrow(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF
		  ("lx_interval acos(const lx_interval& x)"));

    l_interval xl;
    interval z;
    xl = x;
    z = xl;

    real supabsz = Sup(abs(z)),
	infz = Inf(z),
	supz = Sup(z);

    pi = Pi_lx_interval();
    pihalbe = pi;
    times2pown(pihalbe,-1); // pihalbe = pi/2;

    if (infz == supz && supz == 0)
	y = pihalbe;
    else 
	if (infz == supz && supabsz == 1)
	{
	    if (supz == 1.0)
		y = 0;
	    else y = pi;
	}
	else
	{
	    if (0<=z)
		y = pihalbe - atan(x/sqrt1mx2(x));
	    else 
	    {
		y = x;
		neg = Sup(z)<0;
		if (neg) y = -y;
		if (supabsz < 0.25)
		    y = pihalbe - atan(y/sqrt1mx2(y));
		else 
		    y = atan(sqrt1mx2(y)/y);
		if (neg) y = pi - y;
	    }
	}
    stagprec = stagsave; // Restore the old stagprec value
    y = adjust(y); 

    return y;
} // acos(x)

// ------------------- acos(x) ----------------------------------------

lx_interval acot(const lx_interval& x)
// codomain of acot(x): (0,pi);
// acot(x) is continuous at x=0;
// Blomquist, 01.08.07; 
{
    int stagsave = stagprec,
	stagmax = 39,
	ex(expo(x)),
	exl;

    if (stagprec > stagmax) 
	stagprec = stagmax;

    lx_interval y,pihalbe,pi;
    bool neg;

    l_interval xl(li_part(x));
    exl = expo_gr(xl);

    pi = Pi_lx_interval();
    pihalbe = pi;
    times2pown(pihalbe,-1); // pihalbe = pi/2;

    if (0<=xl)
	y = pihalbe - atan(x);
    else 
    {
	y = x;
	neg = Sup(xl) < 0;
	if (neg) y = -y;
	if (ex<=-exl) 
	    if (ex<-exl-1580)
		y = pihalbe;
	    else 
		y = pihalbe - atan(y);
	else y = atan(1/y);
	if (neg) y = pi - y;
    }

    stagprec = stagsave; // Restore the old stagprec value
    y = adjust(y); 

    return y;
} // acot(x)


// -------------------------- sinh(...) --------------------------------------

lx_interval sinh(const lx_interval &x)
// Inclusion of sinh(x)
{
   int stagsave = stagprec,
       stagmax = 39,
       ex,exl;
    if (stagprec > stagmax) 
         stagprec = stagmax;
    lx_interval res(0.0),T;
    l_interval xl(li_part(x));
    bool neg;

    ex = expo(x);
    exl = expo_gr(xl);

    if (0<=xl) // Null in x enthalten!
    {
	    T = expm1(-x);
	    res = -T*(1+1/(T+1));
	    times2pown(res,-1);
    }
    else // Null ist nicht in x enthalten!
	if (ex<=-exl)
	{   // |abs(x)| < 1;
	    T = expm1(-x);
	    res = -T*(1+1/(T+1));
	    times2pown(res,-1);
	}
	else 
	{   // |abs(x)| >= 1;
	    T = x;
	    neg = Inf(xl)<0;
	    if (neg) T = -T;

	    if (ex>12-exl)
	    {
		res = exp(T);
		times2pown(res,-1);
		res += l_interval(-0.5,0);
	    }
	    else
	    {
		res = exp(T);
		res = res - 1/res;
		times2pown(res,-1);
	    }
	    if (neg) res = -res;
	}

    stagprec = stagsave; // Restore the old stagprec value
    res = adjust(res);   

    return res;
} // sinh(...)


// -------------------------- cosh(...) ------------------------------------

lx_interval cosh(const lx_interval& x)
// codomain of cosh(x): [1,+infty);
// Blomquist, 07.08.07; 
{
    int stagsave = stagprec,
	stagmax = 39;
    if (stagprec > stagmax) 
	stagprec = stagmax;

    string s;

    lx_interval y(abs(x));
    lx_real Si;
    l_interval xl;
    interval z;

    xl = y;
    z = xl;

    y = exp(y);
    if (Inf(z)>4096)
    {
	times2pown(y,-1);
	y += l_interval(0,0.5);
    }
    else 
    {
	y = y + 1/y;
	times2pown(y,-1);
    }

    stagprec = stagsave; // Restore the old stagprec value
    y = adjust(y);

    Si = Inf(y);
    if (Si<1.0)  y = lx_interval(lx_real(1.0),Sup(y));

    return y;
} // cosh(x)

// -------------------------- tanh(...) ------------------------------------

lx_interval tanh(const lx_interval& x) 
{
// Blomquist, 10.08.2007;
    int stagsave = stagprec,
	stagmax = 39,
	ex(expo(x)),exl;
    if (stagprec > stagmax) 
	stagprec = stagmax;

    lx_interval y(x),t;
    lx_real Iy;
    bool neg;
    l_interval xl(li_part(x));
    exl = expo_gr(xl);

    if (0<=xl)
    {
	times2pown(y,1); // y = 2x;
	y = expm1(-y);   // y = exp(-2x) - 1;
	y = -y/(2+y);
    }
    else 
    {   // 0 nicht in x enthalten.
	if (ex<=-exl) // 0<|x|<1;
	{
	    times2pown(y,1); // y = 2x;
	    y = expm1(-y);   // y = exp(-2x) - 1;
	    y = -y/(2+y);
	}
	else
	{   // |x| >= 1;
	    neg = Sup(xl)<0;
	    if (neg) y = -y; // Inf(y)>0, bzw. y = abs(x);
	    Iy = Inf(y);
	    ex = expo(Iy);  exl = expo_gr(lr_part(Iy));
	    if (ex >= -exl+13)
		y = lx_interval(Inf(One_m_lx_interval()),lx_real(1));
	    else
	    {
		times2pown(y,1); // y = 2x;
		y = exp(-y);     // y = exp(-2x);
		y = (1-y) / (1+y);
	    }
	    if (neg) y = - y;
	}
    }
 
    stagprec = stagsave; // Restore the old stagprec value
    y = adjust(y); 

    return y;
} // tanh(x)

// -------------------------- coth(...) ------------------------------------

lx_interval coth(const lx_interval& x) throw(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF)
{
// Blomquist, 11.08.2007;
    int stagsave = stagprec,
	stagmax = 39,
	ex(expo(x)),exl;
    if (stagprec > stagmax) 
	stagprec = stagmax;

    lx_interval y(x),t;
    lx_real Iy;
    bool neg;
    l_interval xl(li_part(x));
    exl = expo_gr(xl);

    y = x;
    if (0<=xl) 
	cxscthrow(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF
		  ("lx_interval coth(const lx_interval& x)"));
    neg = Sup(xl)<0;
    if (neg) y = -y; // Inf(y)>0;

    if (ex<=-exl) // 0<|x|<1;
    {
	times2pown(y,1); // y = 2x;
	y = expm1(-y);   // y = exp(-2x) - 1;
	y = -(2+y)/y;
    }
    else
    {   // |x| >= 1;
	Iy = Inf(y);
	ex = expo(Iy);  exl = expo_gr(lr_part(Iy));

	if (ex>=-exl+13)
	    y = lx_interval(lx_real(1),Sup(One_p_lx_interval()));
	else
	{
	    times2pown(y,1); // y = 2x;
	    y = exp(-y);     // y = exp(-2x);
	    y = (1+y) / (1-y);
	}
    }
    if (neg) y = - y;
 
    stagprec = stagsave; // Restore the old stagprec value
    y = adjust(y); 

    return y;
} // coth(x)

// -------------------------- sqrt(1+x)-1 ------------------------------------

lx_interval sqrtp1m1(const lx_interval& x) throw()
// sqrtp1m1(x) calculates an inclusion of sqrt(x+1)-1;
// Blomquist, 25.08.07;
{
    int stagsave=stagprec, 
	stagmax = 30, 
	ex(expo(x)),exl;
    stagprec++;
    if (stagprec>stagmax) stagprec = stagmax;

    lx_interval y(0),tmp;
    l_interval xl(li_part(x));
    interval z;
    const real c = 0.1;

    exl = expo_gr(xl);
    if (exl>-1000000) // x != 0;
    {
	if (ex>-exl+500) 
	    y = sqrt(1+x) - 1;
	else
	{
	    xl = x;  // xl is an inclusion of x,
	    z = xl;  // and z is an inclusion of x;
	    tmp = x+1;
	    y = (z<=interval(-c,c))? x/(sqrt(tmp)+1) : sqrt(tmp)-1;
	}
    }
    stagprec = stagsave;
    y = adjust(y);
    return y; 
} // sqrtp1m1

// -------------------------- asinh(x) ------------------------------------

lx_interval asinh(const lx_interval &x) throw()
// Inclusion of asinh(x)
{
   int stagsave = stagprec,
       stagmax = 39,
       ex,exl;
    if (stagprec > stagmax) 
         stagprec = stagmax;
    lx_interval res(0.0),u;
    lx_real S1,S2;
    l_interval xl(li_part(x));
    interval z;
    bool neg;

    ex = expo(x);
    exl = expo_gr(xl);

    if (0<=xl) // Null in x enthalten!
    {
	xl = x; // Falls hier Fehlermeldung, ist das OK,
	z = xl; // denn sonst ist x viel zu breit!!
	res = asinh(z); // Bis hier OK!!!
    }
    else // Null ist nicht in x enthalten!
    {
	neg = (Inf(xl)<0);
	u = (neg)? -x : x; // Inf(u)>0

	if (ex<=-exl-790)
	{   
	    res = u;
	    S2 = Sup(res);
	    times2pown(u,-1);
	    S1 = Inf( (1-u)*(1+u)*res );
	    res = lx_interval(S1,S2);
	}
	else 
	    res = (ex<=-exl)? lnp1(u+sqrtp1m1(sqr(u))) : ln(u+sqrt1px2(u));
	if (neg) res = -res;
    }

    stagprec = stagsave; // Restore the old stagprec value
    res = adjust(res);   

    return res;
} // asinh(...)
 
// -------------------------- acosh(...) -------------------------------

lx_interval acosh(const lx_interval &x) 
                                throw(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF)
// Inclusion of acosh(x)
{
   int stagsave = stagprec,
       stagmax = 39,
       ex,exl;
    if (stagprec > stagmax) 
         stagprec = stagmax;
    lx_interval res(0.0),x_1;
    lx_real S1,S2;

    if (Inf(x)<1) 
	cxscthrow(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF
		  ("lx_interval acosh(const lx_interval& x)"));
    x_1 = x-1;
    ex = expo(x_1);
    l_interval x_1l(li_part(x_1));
    exl = expo_gr(x_1l);

    if (exl>-100000)
	if (ex <= -exl-1600) 
	{
	    res = x;
	    times2pown(res,1);   // res = 2x;
	    res = sqrt(res - 2); // res = sqrt(2x - 2);
	    S2 = Sup(res);                       
	    res = res * (2-x); // Dieser Trick spart Division durch 12 !!
	    S1 = Inf(res);
	    res = lx_interval(S1,S2);
	}
	else 
	    res = (ex<=-exl)? lnp1( x_1 + sqrt(x_1*(2+x_1)) ) :
		              ln( x+sqrtx2m1(x) );
    stagprec = stagsave; // Restore the old stagprec value
    res = adjust(res);   

    return res;
} // acosh(...)

// ------------------------- acoshp1(...) -----------------------------

lx_interval acoshp1(const lx_interval &x) 
                                throw(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF)
// Inclusion of acosh(1+x);
// Blomquist, 31.08.2007;
{
   int stagsave = stagprec,
       stagmax = 39,
       ex,exl;
    if (stagprec > stagmax) 
         stagprec = stagmax;
    lx_interval res(0.0);
    lx_real S1,S2;

    if (Inf(x)<0) 
	cxscthrow(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF
		  ("lx_interval acoshp1(const lx_interval& x)"));
    ex = expo(x);
    l_interval xl(li_part(x));
    exl = expo_gr(xl);

    if (exl>-100000)
	if (ex <= -exl-1600) 
	{
	    res = x;
	    times2pown(res,1);   // res = 2x;
	    res = sqrt(res); // res = sqrt(2x);
	    S2 = Sup(res);                       
	    res = res * (1-x); 
	    S1 = Inf(res);
	    res = lx_interval(S1,S2);
	}
	else 
	    if (ex <= -exl)
		res = lnp1( x + sqrt(x*(2+x)) );
	    else  // res = acosh(1+x);
	    {
		res = 1+x;
		res = ln(res + sqrtx2m1(res));
	    }

    stagprec = stagsave; // Restore the old stagprec value
    res = adjust(res);   

    return res;
} // acoshp1(...)

// ------------------------ atanh(...) ----------------------------------

lx_interval atanh(const lx_interval &x) 
                                 throw(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF)
// Inclusion of atanh(x)
{
   int stagsave = stagprec,
       stagmax = 39,
       ex,exl;
    if (stagprec > stagmax) 
         stagprec = stagmax;
    lx_interval res(0.0);

    if (Inf(x)<=-1 || Sup(x)>=1) 
	cxscthrow(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF
		              ("lx_interval atanh(const lx_interval& x)"));
    l_interval xl(li_part(x));

    ex = expo(x);
    exl = expo_gr(xl);

    if (0<=xl) // Null in x enthalten!
    {
	res = x/(1-x);
	times2pown(res,1);
	res = lnp1(res);
    }
    else // Null ist nicht in x enthalten!
	if (ex <= -2 - exl)
	{  
	    res = x/(1-x);
	    times2pown(res,1);
	    res = lnp1(res);
	}
	else 
	    res = ln( (1.0+x)/(1.0-x) );
    times2pown(res,-1);

    stagprec = stagsave; // Restore the old stagprec value
    res = adjust(res);   

    return res;
} // atanh(...)

// ------------------------ atanh1m(...) ---------------------------------

lx_interval atanh1m(const lx_interval &x) 
                                 throw(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF)
// Inclusion of atanh(1-x); 0<x<2;
// Blomquist, 09.09.2007;
{
   int stagsave = stagprec,
       stagmax = 39;
    if (stagprec > stagmax) 
         stagprec = stagmax;
    lx_interval res(0.0),u;
    lx_real S1,S2;

    if (Inf(x)<=0 || Sup(x)>=2) 
	cxscthrow(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF
		              ("lx_interval atanh1m(const lx_interval& x)"));
    res = ln(2/x-1);
    times2pown(res,-1);

    stagprec = stagsave; // Restore the old stagprec value
    res = adjust(res);   

    return res;
}  // atanh1m(...)

// ------------------------ atanhm1p(...) ------------------------------

lx_interval atanhm1p(const lx_interval &x) throw()
// Inclusion of atanh(-1+x) , 0<x<2;
{
   int stagsave = stagprec,
       stagmax = 39;
    if (stagprec > stagmax) 
         stagprec = stagmax;
    lx_interval res;

    res = -atanh1m(x);

    stagprec = stagsave; // Restore the old stagprec value
    res = adjust(res);   

    return res;
} // atanhm1p

// ------------------------ acoth(...) ---------------------------------

lx_interval acoth(const lx_interval& x) 
                             throw(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF)
//  Blomquist, 08.09.2007;
{
    int stagsave = stagprec,
	stagmax = 39;
    if (stagprec > stagmax) stagprec = stagmax;
    lx_interval res,u;
    l_interval xl(li_part(x));

    res = lx_interval(0,l_interval(-1,1));
    if ( (Inf(x) <= res) || ((Sup(x)) <= res) || res <= x)
        cxscthrow(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF
                              ("lx_interval acoth(const lx_interval& x)"));
    bool neg;
    neg = Inf(xl)<0;
    if (neg) u = -x;
    else u = x; // u = [u1,u2], u1 > 1;

    res = lnp1(2/(u-1));
    times2pown(res,-1); 
    if (neg) res = -res; 

    stagprec = stagsave;
    res = adjust(res);
 
    return res;
} // acoth

// ------------------------ acoth(1+x) ---------------------------------

lx_interval acothp1(const lx_interval& x) 
                             throw(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF)
//  arcoth(1+x), x>0; Blomquist, 09.09.2007;
{
    int stagsave = stagprec,
	stagmax = 39;
    if (stagprec > stagmax) stagprec = stagmax;
    lx_interval res;
    l_interval xl(li_part(x));

    if (Inf(xl) <= 0.0)
        cxscthrow(ERROR_LINTERVAL_STD_FKT_OUT_OF_DEF
                               ("lx_interval acothp1(const lx_interval& x)"));
    res = lnp1(2/x);
    times2pown(res,-1); 

    stagprec = stagsave;
    res = adjust(res);
 
    return res;
} // acothp1

// ------------------------ atanhm1p(...) ------------------------------

lx_interval acothm1m(const lx_interval &x) throw()
// Inclusion of acoth(-1-x) , x>0;
{
   int stagsave = stagprec,
       stagmax = 39;
    if (stagprec > stagmax) 
         stagprec = stagmax;
    lx_interval res;

    res = -acothp1(x);

    stagprec = stagsave; // Restore the old stagprec value
    res = adjust(res);   

    return res;
} // acothm1m

// ------------------------ sqrt(x^2+y^2) ------------------------------

lx_interval sqrtx2y2(const lx_interval &x, const lx_interval &y) throw()
// Inclusion of sqrt(x^2+y^2);
// Blomquist, 12.09.2007;
{
    int stagsave = stagprec,
	stagmax = 30,
	exx,exxl,exy,exyl;
    if (stagprec > stagmax) 
	stagprec = stagmax;
    lx_interval res,ax,ay;
    lx_real R;

    ax = abs(x);    ay = abs(y);
    if ( Sup(ay) > Sup(ax) ) 
    {
	res = ax; ax = ay; ay = res;
    }   // Sup(ax) >= Sup(ay);
    if (0<=ax) // 0 is included by ax;
	res = sqrt(sqr(ax) + sqr(ay));
    else  // 0 not included by ax;
    {
	R = Sup(ay);
	exy = expo(R);
	exyl = expo_gr( lr_part(R) );

	R = Inf(ax);
	exx = expo(R);
	exxl = expo_gr( lr_part(R) );

	if (exyl>-100000) // Y2 > 0;
	{
	    exxl = exxl-exyl-1051;   // Without integer-Overflow 
            exx = add_int(exx,exxl); // Integer-Overflow only in rare cases
	    res = (exy<exx)? 
		ax * lx_interval(lx_real(1.0),Sup(One_p_lx_interval())) :
		ax * sqrt1px2(ay/ax);
	}
	else // Y2 = 0;
	    res = ax;
    }

    stagprec = stagsave; // Restore the old stagprec value
    res = adjust(res);   

    return res;
}  // sqrtx2y2(...)


// ---------------------- ln(sqrt(x^2+y^2)) ----------------------------

lx_interval ln_sqrtx2y2(const lx_interval &x, const lx_interval &y) throw()
// Inclusion of ln(sqrt(x^2+y^2));
// Blomquist, 15.09.2007;
{
    int stagsave = stagprec,
	stagmax = 30,
	ex_y,N,N2;
    if (stagprec > stagmax) 
	stagprec = stagmax;
    lx_interval res,ax,ay;
    lx_real Rx,Ry;
    bool scal;

    ax = abs(x);    ay = abs(y);
    Rx = Sup(ax);   Ry = Sup(ay);
    if (Ry>Rx) 
    {
	Rx = Ry; // Rx ist maximaler Wert
	res = ay;   ay = ax;   ax = res; // ax ist maximaler Wert
    }

    if (Rx>100) res = ln( sqrtx2y2(ax,ay) );
    else 
	if (Rx<0.25)
	    res = ln( sqrtx2y2(ax,ay) );
	else
	{
	    // Calculating:  0.5*ln(x*x + y*y)
            //             = 0.5*ln(1 + (x*x + y*y - 1))
            //             = 0.5*ln(1 + ((x-1)*(x+1) + y*y))
	    if (ay==0.0)
		res = ln(ax);  // Without Overflow!
	    else
	    {
		if (Inf(ay)==0.0) ex_y = expo(Sup(ay));
		else ex_y = expo(Inf(ay));

		scal = ex_y<-1073741312;
		if (scal)
		{
		    N = -ex_y-1073741313;
		    N2 = add_int(N,N);
		    res = (ax-1)*(ax+1);
		    times2pown(res,N2);
		    times2pown(ay,N);
		    res = res + ay*ay;
		    times2pown(res,-N2);
		    res = lnp1(res);
		}
		else
		    res = lnp1((ax-1)*(ax+1) + ay*ay);
		times2pown(res,-1);
	    }
	}

    stagprec = stagsave; // Restore the old stagprec value
    res = adjust(res);   

    return res;
}  // ln_sqrtx2y2(...)

// -------------------------- n-th root --------------------------------

lx_interval sqrt(const lx_interval& x, int n) throw()
{   // Inclusion of the n-th root, based on the pow(...) function
    int stagsave = stagprec,
	stagmax = 39;
    if (stagprec>stagmax) stagprec = stagmax;

    lx_interval res;
    lx_real Sx;
    int k(n);
    if (k<2) k=2;
    if (eq_zero(Inf(x)))
    { 
	Sx = Sup(x);
	if (gr_zero(Sx))
	{
	    res = pow(lx_interval(Sx),1/lx_interval(k));
	    res = lx_interval(lx_real(0),Sup(res));
	}
	else res = 0.0;
    }
    else
	res = pow(x,1/lx_interval(k));
    stagprec = stagsave; // Restore the old stagprec value
    res = adjust(res); 

    return res;
}

// -------------------------- Konstanten -------------------------------

static real CXSC_Pi[40]; // CXSC_Pi[0], ... ,CXSC_Pi[39]
static bool CXSC_Pi_initialized = false;

lx_interval Pi_lx_interval() throw()
// Inclusion of Pi, Blomquist, 15.06.2007;
{
    l_interval y;
    int stagsave = stagprec,
	stagmax = 39;
    if (!CXSC_Pi_initialized)
    {
	std::string str;
	std::cout << SaveOpt;
	std::cout << Hex;
	str = "+1921FB54442D18e7FC";
	str >> CXSC_Pi[0];
	str = "+11A62633145C07e7C6";
	str >> CXSC_Pi[1];
	str = "-1F1976B7ED8FBCe78E";
	str >> CXSC_Pi[2];
	str = "+14CF98E804177De758";
	str >> CXSC_Pi[3];
	str = "+131D89CD9128A5e722";
	str >> CXSC_Pi[4];
	str = "+10F31C6809BBDFe6E8";
	str >> CXSC_Pi[5];
	str = "+1519B3CD3A431Be6B1";
	str >> CXSC_Pi[6];
	str = "+18158536F92F8Ae67A";
	str >> CXSC_Pi[7];
	str = "+1BA7F09AB6B6A9e642";
	str >> CXSC_Pi[8];
	str = "-1EDD0DBD2544CFe60A";
	str >> CXSC_Pi[9];
	str = "+179FB1BD1310BAe5D3";
	str >> CXSC_Pi[10];
	str = "+1A637ED6B0BFF6e59D";
	str >> CXSC_Pi[11];
	str = "-1A485FCA40908Ee566";
	str >> CXSC_Pi[12];
	str = "-1E501295D98169e52F";
	str >> CXSC_Pi[13];
	str = "-1160DBEE83B4E0e4F9";
	str >> CXSC_Pi[14];
	str = "-19B6D799AE131Ce4C1";
	str >> CXSC_Pi[15];
	str = "+16CF70801F2E28e48B";
	str >> CXSC_Pi[16];
	str = "+163BF0598DA483e455";
	str >> CXSC_Pi[17];
	str = "+1871574E69A459e41F";
	str >> CXSC_Pi[18];
	str = "-15C0B6CC281F27e3E3";
	str >> CXSC_Pi[19];
	str = "+15D23DCA3AD962e3AD";
	str >> CXSC_Pi[20];
	str = "-1CE8654EFBD56Ae376";
	str >> CXSC_Pi[21];
	str = "-1184AB5BE23DA6e33F";
	str >> CXSC_Pi[22];
	str = "+166D670C354E4Be309";
	str >> CXSC_Pi[23];
	str = "-10D9FEC3A2E4FEe2D3";
	str >> CXSC_Pi[24];
	str = "+1943042F86520Ce29C";
	str >> CXSC_Pi[25];
	str = "-1B9D1C931C41C6e265";
	str >> CXSC_Pi[26];
	str = "-188D3E7F179FC6e22D";
	str >> CXSC_Pi[27];
	str = "-1361F1744FE176e1F7";
	str >> CXSC_Pi[28];
	str = "+1F6B8ABBE0DE99e1C0";
	str >> CXSC_Pi[29];
	str = "-169B10EA1A04B5e18A";
	str >> CXSC_Pi[30];
	str = "-14FD1CF8CD56D0e154";
	str >> CXSC_Pi[31];
	str = "-18AB54A8D7516Fe11E";
	str >> CXSC_Pi[32];
	str = "+186263E8144056e0E7";
	str >> CXSC_Pi[33];
	str = "-1AE34AEAAA77A5e0B0";
	str >> CXSC_Pi[34];
	str = "+16998B8682283De07A";
	str >> CXSC_Pi[35];
	str = "+19D42A90D5EF8Ee042";
	str >> CXSC_Pi[36];
	str = "+174C9D9F70A08Be00C";
	str >> CXSC_Pi[37];
	str = "+100000000000DBe000";
	str >> CXSC_Pi[38];
	str = "+100000000000DCe000";
	str >> CXSC_Pi[39];

	CXSC_Pi_initialized = true;
	std::cout << RestoreOpt;
    } 
    stagprec = stagmax;
    y = adjust(l_interval(0));

    for (int i=0; i<=stagmax; i++)
	y[i+1] = CXSC_Pi[i];

    stagprec = stagsave;
    y = adjust(y);

    return lx_interval(-1020,y);
} // Pi


static real CXSC_Ln2[40]; // CXSC_Ln2[0], ... ,CXSC_Ln2[39]
static bool CXSC_Ln2_initialized = false;

lx_interval Ln2_lx_interval() throw()
// Inclusion of ln(2), Blomquist, 15.06.2007;
{
    l_interval y;
    int stagsave = stagprec,
	stagmax = 39;
    if (!CXSC_Ln2_initialized)
    {
	std::string str;
	std::cout << SaveOpt;
	std::cout << Hex;
	str = "+162E42FEFA39EFe7FC";
	str >> CXSC_Ln2[0];
	str = "+1ABC9E3B39803Fe7C5";
	str >> CXSC_Ln2[1];
	str = "+17B57A079A1934e78E";
	str >> CXSC_Ln2[2];
	str = "-1ACE93A4EBE5D1e758";
	str >> CXSC_Ln2[3];
	str = "-123A2A82EA0C24e722";
	str >> CXSC_Ln2[4];
	str = "+1D881B7AEB2615e6EB";
	str >> CXSC_Ln2[5];
	str = "+19552FB4AFA1B1e6B5";
	str >> CXSC_Ln2[6];
	str = "+1DA5D5C6B82704e67C";
	str >> CXSC_Ln2[7];
	str = "+14427573B29117e645";
	str >> CXSC_Ln2[8];
	str = "-191F6B05A4D7A7e60F";
	str >> CXSC_Ln2[9];
	str = "-1DB5173AE53426e5D9";
	str >> CXSC_Ln2[10];
	str = "+11317C387EB9EBe5A1";
	str >> CXSC_Ln2[11];
	str = "-190F13B267F137e56B";
	str >> CXSC_Ln2[12];
	str = "+16FA0EC7657F75e535";
	str >> CXSC_Ln2[13];
	str = "-1234C5E1398A6Be4FF";
	str >> CXSC_Ln2[14];
	str = "+1195EBBF4D7A70e4C8";
	str >> CXSC_Ln2[15];
	str = "+18192432AFD0C4e492";
	str >> CXSC_Ln2[16];
	str = "-1A1BE38BA4BA4De45C";
	str >> CXSC_Ln2[17];
	str = "-1D7860151CFC06e422";
	str >> CXSC_Ln2[18];
	str = "+19423F6B7F720Ce3EC";
	str >> CXSC_Ln2[19];
	str = "+10D30F88FE551Ae3B5";
	str >> CXSC_Ln2[20];
	str = "+1772B4EB6FE0F8e37E";
	str >> CXSC_Ln2[21];
	str = "-17AA0B477087B0e347";
	str >> CXSC_Ln2[22];
	str = "+1672C2E8C0EEBBe30C";
	str >> CXSC_Ln2[23];
	str = "+1C4C872E4A1F3Ae2D6";
	str >> CXSC_Ln2[24];
	str = "-1A970C65986667e2A0";
	str >> CXSC_Ln2[25];
	str = "-18CD36365759DAe26A";
	str >> CXSC_Ln2[26];
	str = "+1A1E0BD1D6095De231";
	str >> CXSC_Ln2[27];
	str = "+12B34D999AB252e1FA";
	str >> CXSC_Ln2[28];
	str = "-1912AC700EB43De1C4";
	str >> CXSC_Ln2[29];
	str = "-1B8BEFC5924FF5e18E";
	str >> CXSC_Ln2[30];
	str = "-180C2AE79DBFADe156";
	str >> CXSC_Ln2[31];
	str = "-17D195E5A6D545e120";
	str >> CXSC_Ln2[32];
	str = "-1743270F423129e0EA";
	str >> CXSC_Ln2[33];
	str = "+189E6DB6303659e0B2";
	str >> CXSC_Ln2[34];
	str = "-1F0E11945C9A4Ae07C";
	str >> CXSC_Ln2[35];
	str = "+18DAFA85A8C283e046";
	str >> CXSC_Ln2[36];
	str = "+13062D3458B6CFe00F";
	str >> CXSC_Ln2[37];
	str = "-10000000000C9Be000";
	str >> CXSC_Ln2[38];
	str = "-10000000000C9Ae000";
	str >> CXSC_Ln2[39];

	CXSC_Ln2_initialized = true;
	std::cout << RestoreOpt;
    } 
    stagprec = stagmax;
    y = adjust(l_interval(0));

    for (int i=0; i<=stagmax; i++)
	y[i+1] = CXSC_Ln2[i];

    stagprec = stagsave;
    y = adjust(y);

    return lx_interval(-1022,y);
} // ln(2)

static real CXSC_Ln10[40]; // CXSC_Ln10[0], ... ,CXSC_Ln10[39]
static bool CXSC_Ln10_initialized = false;

lx_interval Ln10_lx_interval() throw()
// Inclusion of ln(10), Blomquist, 15.06.2007;
{
    l_interval y;
    int stagsave = stagprec,
	stagmax = 39;
    if (!CXSC_Ln10_initialized)
    {
	std::string str;
	std::cout << SaveOpt;
	std::cout << Hex;
	str = "+126BB1BBB55516e7FD";
	str >> CXSC_Ln10[0];
	str = "-1F48AD494EA3E9e7C7";
	str >> CXSC_Ln10[1];
	str = "-19EBAE3AE0260Ce791";
	str >> CXSC_Ln10[2];
	str = "-12D10378BE1CF1e75B";
	str >> CXSC_Ln10[3];
	str = "+10403E05AE52C6e725";
	str >> CXSC_Ln10[4];
	str = "-1FA509CAFDF466e6ED";
	str >> CXSC_Ln10[5];
	str = "-1C79A1FE9D0795e6B7";
	str >> CXSC_Ln10[6];
	str = "+1058C448308218e681";
	str >> CXSC_Ln10[7];
	str = "-1D250470877BFDe64A";
	str >> CXSC_Ln10[8];
	str = "-1AE92987D3075De612";
	str >> CXSC_Ln10[9];
	str = "-1D5CDBB8626956e5DC";
	str >> CXSC_Ln10[10];
	str = "-13C4F27CE0410Ae5A6";
	str >> CXSC_Ln10[11];
	str = "+1B3AC12ACF1BE9e570";
	str >> CXSC_Ln10[12];
	str = "+1161BB49D219C8e53A";
	str >> CXSC_Ln10[13];
	str = "-110D6613293728e504";
	str >> CXSC_Ln10[14];
	str = "+142163A4CDA351e4CC";
	str >> CXSC_Ln10[15];
	str = "+1E2713D6C22C16e494";
	str >> CXSC_Ln10[16];
	str = "-15090EF85CB0ADe45B";
	str >> CXSC_Ln10[17];
	str = "-1C5B3E859F876Ee424";
	str >> CXSC_Ln10[18];
	str = "-1C0D54B14459D9e3EC";
	str >> CXSC_Ln10[19];
	str = "+1AB685CD44E2C3e3B5";
	str >> CXSC_Ln10[20];
	str = "+1A47ECB26978C6e37E";
	str >> CXSC_Ln10[21];
	str = "-15812716B8AD41e347";
	str >> CXSC_Ln10[22];
	str = "-16047E37E81868e311";
	str >> CXSC_Ln10[23];
	str = "+1E14126A45765De2DA";
	str >> CXSC_Ln10[24];
	str = "-10ECBE631205C0e2A3";
	str >> CXSC_Ln10[25];
	str = "-15A485363BE7D4e26C";
	str >> CXSC_Ln10[26];
	str = "-1DEDE455922FF8e234";
	str >> CXSC_Ln10[27];
	str = "-1C202C3AE8B719e1FE";
	str >> CXSC_Ln10[28];
	str = "-148E3DB9B96D03e1C7";
	str >> CXSC_Ln10[29];
	str = "+1E3795D1008FE3e191";
	str >> CXSC_Ln10[30];
	str = "-13C992BD5AD855e158";
	str >> CXSC_Ln10[31];
	str = "-152096175A0882e122";
	str >> CXSC_Ln10[32];
	str = "+1BB0274A1CB072e0EB";
	str >> CXSC_Ln10[33];
	str = "-1D6A3FC0087494e0B4";
	str >> CXSC_Ln10[34];
	str = "+1AD6BFBFFD821Ce07E";
	str >> CXSC_Ln10[35];
	str = "-17D6CD3EE64A79e048";
	str >> CXSC_Ln10[36];
	str = "-166DC44198DC68e010";
	str >> CXSC_Ln10[37];
	str = "-100000000012D2e000";
	str >> CXSC_Ln10[38];
	str = "-100000000012D1e000";
	str >> CXSC_Ln10[39];

	CXSC_Ln10_initialized = true;
	std::cout << RestoreOpt;
    } 
    stagprec = stagmax;
    y = adjust(l_interval(0));

    for (int i=0; i<=stagmax; i++)
	y[i+1] = CXSC_Ln10[i];

    stagprec = stagsave;
    y = adjust(y);

    return lx_interval(-1021,y);
} // ln(10)

static real CXSC_Pir[40]; // CXSC_Pir[0], ... ,CXSC_Pir[39]
static bool CXSC_Pir_initialized = false;

lx_interval Pir_lx_interval() throw()
// Inclusion of 1/Pi, Blomquist, 15.06.2007;
{
    l_interval y;
    int stagsave = stagprec,
	stagmax = 39;
    if (!CXSC_Pir_initialized)
    {
	std::string str;
	std::cout << SaveOpt;
	std::cout << Hex;
	str = "+145F306DC9C883e7FC";
	str >> CXSC_Pir[0];
	str = "-16B01EC5417056e7C6";
	str >> CXSC_Pir[1];
	str = "-16447E493AD4CEe790";
	str >> CXSC_Pir[2];
	str = "+1E21C820FF28B2e75A";
	str >> CXSC_Pir[3];
	str = "-1508510EA79237e723";
	str >> CXSC_Pir[4];
	str = "+1B8E909374B802e6EB";
	str >> CXSC_Pir[5];
	str = "-1B6D115F62E6DEe6B5";
	str >> CXSC_Pir[6];
	str = "-180F10A71A76B3e67E";
	str >> CXSC_Pir[7];
	str = "+1CFBA208D7D4BBe647";
	str >> CXSC_Pir[8];
	str = "-12EDEC598E3F65e60F";
	str >> CXSC_Pir[9];
	str = "-1741037D8CDC54e5D8";
	str >> CXSC_Pir[10];
	str = "+1CC1A99CFA4E42e5A2";
	str >> CXSC_Pir[11];
	str = "+17E2EF7E4A0EC8e56B";
	str >> CXSC_Pir[12];
	str = "-1DA00087E99FC0e52F";
	str >> CXSC_Pir[13];
	str = "-10D0EE74A5F593e4F9";
	str >> CXSC_Pir[14];
	str = "+1F6D367ECF27CBe4C1";
	str >> CXSC_Pir[15];
	str = "+136E9E8C7ECD3De488";
	str >> CXSC_Pir[16];
	str = "-100AE9456C229Ce452";
	str >> CXSC_Pir[17];
	str = "-141A0E84C2F8C6e419";
	str >> CXSC_Pir[18];
	str = "-10EB5ADA2B2809e3E0";
	str >> CXSC_Pir[19];
	str = "-10277039517BD5e3AA";
	str >> CXSC_Pir[20];
	str = "+198237E3DB5D60e36E";
	str >> CXSC_Pir[21];
	str = "-1E6087BECA1794e338";
	str >> CXSC_Pir[22];
	str = "+1DA9E391615EE6e301";
	str >> CXSC_Pir[23];
	str = "+1B086599855F15e2C9";
	str >> CXSC_Pir[24];
	str = "-17E5EFDC8009E0e293";
	str >> CXSC_Pir[25];
	str = "+135CC9CC418185e25B";
	str >> CXSC_Pir[26];
	str = "+156CA73A8C960Ee225";
	str >> CXSC_Pir[27];
	str = "+13DE04635A3E21e1EE";
	str >> CXSC_Pir[28];
	str = "-18F260C88C5FDBe1B7";
	str >> CXSC_Pir[29];
	str = "-157CA63B89746Ae181";
	str >> CXSC_Pir[30];
	str = "+1CA6DDAF44D157e149";
	str >> CXSC_Pir[31];
	str = "+19053EA5FF0705e111";
	str >> CXSC_Pir[32];
	str = "+1FBF19F419616Fe0DA";
	str >> CXSC_Pir[33];
	str = "+13E60C9F6EF0CFe0A3";
	str >> CXSC_Pir[34];
	str = "+126EF6B1E5EF8Ae06D";
	str >> CXSC_Pir[35];
	str = "-18BC1946A1B01Ce034";
	str >> CXSC_Pir[36];
	str = "-12780EDE6F8384e000";
	str >> CXSC_Pir[37];
	str = "+10000000000000e000";
	str >> CXSC_Pir[38];
	str = "+10000000000001e000";
	str >> CXSC_Pir[39];

	CXSC_Pir_initialized = true;
	std::cout << RestoreOpt;
    } 
    stagprec = stagmax;
    y = adjust(l_interval(0));

    for (int i=0; i<=stagmax; i++)
	y[i+1] = CXSC_Pir[i];

    stagprec = stagsave;
    y = adjust(y);

    return lx_interval(-1023,y);
} // 1/Pi

static real CXSC_SqrtPi[40]; // CXSC_SqrtPi[0], ... ,CXSC_SqrtPi[39]
static bool CXSC_SqrtPi_initialized = false;

lx_interval SqrtPi_lx_interval() throw()
// Inclusion of sqrt(Pi), Blomquist, 15.06.2007;
{
    l_interval y;
    int stagsave = stagprec,
	stagmax = 39;
    if (!CXSC_SqrtPi_initialized)
    {
	std::string str;
	std::cout << SaveOpt;
	std::cout << Hex;
	str = "+1C5BF891B4EF6Be7FC";
	str >> CXSC_SqrtPi[0];
	str = "-1618F13EB7CA89e7C6";
	str >> CXSC_SqrtPi[1];
	str = "-1B1F0071B7AAE4e78E";
	str >> CXSC_SqrtPi[2];
	str = "-1389B5A46BDFE8e757";
	str >> CXSC_SqrtPi[3];
	str = "-160AF5C5C89448e721";
	str >> CXSC_SqrtPi[4];
	str = "-14835F07122994e6E5";
	str >> CXSC_SqrtPi[5];
	str = "+1CEC283C18EE8Fe6AF";
	str >> CXSC_SqrtPi[6];
	str = "-13ADEBB9223CA8e678";
	str >> CXSC_SqrtPi[7];
	str = "+1454912430D291e642";
	str >> CXSC_SqrtPi[8];
	str = "-1E8B2345020EF6e60C";
	str >> CXSC_SqrtPi[9];
	str = "-17262982556291e5D5";
	str >> CXSC_SqrtPi[10];
	str = "+1196FA9B140CABe59E";
	str >> CXSC_SqrtPi[11];
	str = "-175EEE59D91D39e568";
	str >> CXSC_SqrtPi[12];
	str = "+1789268B7D9D48e52D";
	str >> CXSC_SqrtPi[13];
	str = "+17162E2F06B89Ce4F7";
	str >> CXSC_SqrtPi[14];
	str = "+1EC9C08F40A3DBe4C0";
	str >> CXSC_SqrtPi[15];
	str = "+1B6048DD0729E2e48A";
	str >> CXSC_SqrtPi[16];
	str = "+1471CF4C33FF6Be453";
	str >> CXSC_SqrtPi[17];
	str = "+1D75FBD8B36F94e41D";
	str >> CXSC_SqrtPi[18];
	str = "+16BA59D137CC6Ee3E7";
	str >> CXSC_SqrtPi[19];
	str = "-1FDFA25FB4BFD8e3B1";
	str >> CXSC_SqrtPi[20];
	str = "-1699363F68227Be379";
	str >> CXSC_SqrtPi[21];
	str = "-1BDD2FD4684487e342";
	str >> CXSC_SqrtPi[22];
	str = "+1122B2D8015ED6e30B";
	str >> CXSC_SqrtPi[23];
	str = "-17EB1A81424DE5e2D2";
	str >> CXSC_SqrtPi[24];
	str = "+1C08B42B2EB0E0e29C";
	str >> CXSC_SqrtPi[25];
	str = "-1316DE24F93E9Fe266";
	str >> CXSC_SqrtPi[26];
	str = "+129354F2D42931e230";
	str >> CXSC_SqrtPi[27];
	str = "-1CB7B480D41490e1FA";
	str >> CXSC_SqrtPi[28];
	str = "+1608DE7786C4ABe1C3";
	str >> CXSC_SqrtPi[29];
	str = "-117732A85F48BCe18D";
	str >> CXSC_SqrtPi[30];
	str = "-18BFB034DC2D75e156";
	str >> CXSC_SqrtPi[31];
	str = "+155DAB8C4A398Ee120";
	str >> CXSC_SqrtPi[32];
	str = "+161C9A5BA77FF3e0E8";
	str >> CXSC_SqrtPi[33];
	str = "-1ECF0081DB503Ce0B2";
	str >> CXSC_SqrtPi[34];
	str = "-192FF4749E0FD8e07B";
	str >> CXSC_SqrtPi[35];
	str = "-1B84C9BCD51654e044";
	str >> CXSC_SqrtPi[36];
	str = "-11CF482677D72Fe00A";
	str >> CXSC_SqrtPi[37];
	str = "-10000000000025e000";
	str >> CXSC_SqrtPi[38];
	str = "-10000000000024e000";
	str >> CXSC_SqrtPi[39];

	CXSC_SqrtPi_initialized = true;
	std::cout << RestoreOpt;
    } 
    stagprec = stagmax;
    y = adjust(l_interval(0));

    for (int i=0; i<=stagmax; i++)
	y[i+1] = CXSC_SqrtPi[i];

    stagprec = stagsave;
    y = adjust(y);

    return lx_interval(-1021,y);
} // sqrt(Pi)

static real CXSC_Sqrt2Pi[40]; // CXSC_Sqrt2Pi[0], ... ,CXSC_Sqrt2Pi[39]
static bool CXSC_Sqrt2Pi_initialized = false;

lx_interval Sqrt2Pi_lx_interval() throw()
// Inclusion of sqrt(2*Pi), Blomquist, 15.06.2007;
{
    l_interval y;
    int stagsave = stagprec,
	stagmax = 39;
    if (!CXSC_Sqrt2Pi_initialized)
    {
	std::string str;
	std::cout << SaveOpt;
	std::cout << Hex;
	str = "+140D931FF62706e7FD";
	str >> CXSC_Sqrt2Pi[0];
	str = "-1A6A0D6F814637e7C7";
	str >> CXSC_Sqrt2Pi[1];
	str = "-1311D073060ACEe791";
	str >> CXSC_Sqrt2Pi[2];
	str = "+16000B50DC2F41e758";
	str >> CXSC_Sqrt2Pi[3];
	str = "+16EF75CA45A834e721";
	str >> CXSC_Sqrt2Pi[4];
	str = "+19BDB2B4C39342e6E9";
	str >> CXSC_Sqrt2Pi[5];
	str = "+1F5582E2063EE6e6B2";
	str >> CXSC_Sqrt2Pi[6];
	str = "+183F879BEA150Ce679";
	str >> CXSC_Sqrt2Pi[7];
	str = "-1F1EA3CA289B00e641";
	str >> CXSC_Sqrt2Pi[8];
	str = "-1699CDA77736F9e60A";
	str >> CXSC_Sqrt2Pi[9];
	str = "-11A379D298B55Ee5D1";
	str >> CXSC_Sqrt2Pi[10];
	str = "-1A6DDB0152BA94e59B";
	str >> CXSC_Sqrt2Pi[11];
	str = "-1957E2E58A02FEe564";
	str >> CXSC_Sqrt2Pi[12];
	str = "-1D6160F18E604De52E";
	str >> CXSC_Sqrt2Pi[13];
	str = "+1311860CDF7215e4F5";
	str >> CXSC_Sqrt2Pi[14];
	str = "+12271F44C50274e4BE";
	str >> CXSC_Sqrt2Pi[15];
	str = "-100BF5C5497A21e487";
	str >> CXSC_Sqrt2Pi[16];
	str = "+1E94B6E6AD51E2e44F";
	str >> CXSC_Sqrt2Pi[17];
	str = "-1C910B5F3D27CEe416";
	str >> CXSC_Sqrt2Pi[18];
	str = "+1F266C0EA9E7FBe3E0";
	str >> CXSC_Sqrt2Pi[19];
	str = "+1D84A8782A175De3AA";
	str >> CXSC_Sqrt2Pi[20];
	str = "-1C75F70BEA4BE2e36F";
	str >> CXSC_Sqrt2Pi[21];
	str = "+1ABE52E82F3797e338";
	str >> CXSC_Sqrt2Pi[22];
	str = "-10A542CE87822Be302";
	str >> CXSC_Sqrt2Pi[23];
	str = "+1457EC878576D9e2CC";
	str >> CXSC_Sqrt2Pi[24];
	str = "-1E99158B23E861e296";
	str >> CXSC_Sqrt2Pi[25];
	str = "-178F7C0F0F2130e25F";
	str >> CXSC_Sqrt2Pi[26];
	str = "-12619EF8E11367e223";
	str >> CXSC_Sqrt2Pi[27];
	str = "-1F5C2604AB7BA5e1EC";
	str >> CXSC_Sqrt2Pi[28];
	str = "+13ED3039D91C88e1B2";
	str >> CXSC_Sqrt2Pi[29];
	str = "-19055B38C434DDe17A";
	str >> CXSC_Sqrt2Pi[30];
	str = "+1A7547A2C9CA85e144";
	str >> CXSC_Sqrt2Pi[31];
	str = "-1543DE12F9EBD5e10E";
	str >> CXSC_Sqrt2Pi[32];
	str = "+180AD772E1F1A0e0D8";
	str >> CXSC_Sqrt2Pi[33];
	str = "+1FEF40C66F715Ee0A2";
	str >> CXSC_Sqrt2Pi[34];
	str = "+14C244F317CD0De069";
	str >> CXSC_Sqrt2Pi[35];
	str = "-1A29C32DBBFCAFe030";
	str >> CXSC_Sqrt2Pi[36];
	str = "-1038D4217EB5FCe000";
	str >> CXSC_Sqrt2Pi[37];
	str = "-10000000000001e000";
	str >> CXSC_Sqrt2Pi[38];
	str = "-10000000000000e000";
	str >> CXSC_Sqrt2Pi[39];


	CXSC_Sqrt2Pi_initialized = true;
	std::cout << RestoreOpt;
    } 
    stagprec = stagmax;
    y = adjust(l_interval(0));

    for (int i=0; i<=stagmax; i++)
	y[i+1] = CXSC_Sqrt2Pi[i];

    stagprec = stagsave;
    y = adjust(y);

    return lx_interval(-1021,y);
} // sqrt(2*Pi)

static real CXSC_Sqrt2[40]; // CXSC_Sqrt2[0], ... ,CXSC_Sqrt2[39]
static bool CXSC_Sqrt2_initialized = false;

lx_interval Sqrt2_lx_interval() throw()
// Inclusion of sqrt(2), Blomquist, 15.06.2007;
{
    l_interval y;
    int stagsave = stagprec,
	stagmax = 39;
    if (!CXSC_Sqrt2_initialized)
    {
	std::string str;
	std::cout << SaveOpt;
	std::cout << Hex;
	str = "+16A09E667F3BCDe7FC";
	str >> CXSC_Sqrt2[0];
	str = "-1BDD3413B26456e7C6";
	str >> CXSC_Sqrt2[1];
	str = "+157D3E3ADEC175e790";
	str >> CXSC_Sqrt2[2];
	str = "+12775099DA2F59e758";
	str >> CXSC_Sqrt2[3];
	str = "+160CCE64552BF2e71F";
	str >> CXSC_Sqrt2[4];
	str = "+1821D5C5161D46e6E6";
	str >> CXSC_Sqrt2[5];
	str = "-1C032046F8498Ee6B0";
	str >> CXSC_Sqrt2[6];
	str = "+1EE950BC8738F7e678";
	str >> CXSC_Sqrt2[7];
	str = "-1AC3FDBC64E103e642";
	str >> CXSC_Sqrt2[8];
	str = "+13B469101743A1e60A";
	str >> CXSC_Sqrt2[9];
	str = "+15E3E9CA60B38Ce5D4";
	str >> CXSC_Sqrt2[10];
	str = "+11BC337BCAB1BDe599";
	str >> CXSC_Sqrt2[11];
	str = "-1BBA5DEE9D6E7De563";
	str >> CXSC_Sqrt2[12];
	str = "-1438DD083B1CC4e52D";
	str >> CXSC_Sqrt2[13];
	str = "+1B56A28E2EDFA7e4F7";
	str >> CXSC_Sqrt2[14];
	str = "+1CCB2A634331F4e4C1";
	str >> CXSC_Sqrt2[15];
	str = "-1BD9056876F83Ee48A";
	str >> CXSC_Sqrt2[16];
	str = "-1234FA22AB6BEFe454";
	str >> CXSC_Sqrt2[17];
	str = "+19040CA4A81395e41D";
	str >> CXSC_Sqrt2[18];
	str = "-15249C0BC4082De3E7";
	str >> CXSC_Sqrt2[19];
	str = "+13A02CEBC93E0Ce3B1";
	str >> CXSC_Sqrt2[20];
	str = "+109936AF354A2Ee37B";
	str >> CXSC_Sqrt2[21];
	str = "-1AE4730CBE4908e345";
	str >> CXSC_Sqrt2[22];
	str = "+11B6380826E010e30E";
	str >> CXSC_Sqrt2[23];
	str = "-1CDCAD0CCD5A16e2D5";
	str >> CXSC_Sqrt2[24];
	str = "-1084BC28012BC8e29C";
	str >> CXSC_Sqrt2[25];
	str = "-1C035DDECF8216e265";
	str >> CXSC_Sqrt2[26];
	str = "+18907DEAA070B0e22B";
	str >> CXSC_Sqrt2[27];
	str = "+1FCBDDEA2F7DC3e1F5";
	str >> CXSC_Sqrt2[28];
	str = "+18C41C51757FB0e1BE";
	str >> CXSC_Sqrt2[29];
	str = "-189A5B616B1381e188";
	str >> CXSC_Sqrt2[30];
	str = "+165C417EFF0B88e152";
	str >> CXSC_Sqrt2[31];
	str = "-1627043F832999e11A";
	str >> CXSC_Sqrt2[32];
	str = "+105E5FCA017092e0E3";
	str >> CXSC_Sqrt2[33];
	str = "-187A16D6A8FDCAe0AD";
	str >> CXSC_Sqrt2[34];
	str = "-1838421AE0AE62e072";
	str >> CXSC_Sqrt2[35];
	str = "-165073EB433984e03C";
	str >> CXSC_Sqrt2[36];
	str = "+1F0A42F9DA4A6Ce006";
	str >> CXSC_Sqrt2[37];
	str = "+10000000000002e000";
	str >> CXSC_Sqrt2[38];
	str = "+10000000000003e000";
	str >> CXSC_Sqrt2[39];

	CXSC_Sqrt2_initialized = true;
	std::cout << RestoreOpt;
    } 
    stagprec = stagmax;
    y = adjust(l_interval(0));

    for (int i=0; i<=stagmax; i++)
	y[i+1] = CXSC_Sqrt2[i];

    stagprec = stagsave;
    y = adjust(y);

    return lx_interval(-1021,y);
} // sqrt(2)

static real CXSC_Sqrt3[40]; // CXSC_Sqrt3[0], ... ,CXSC_Sqrt3[39]
static bool CXSC_Sqrt3_initialized = false;

lx_interval Sqrt3_lx_interval() throw()
// Inclusion of sqrt(3), Blomquist, 15.06.2007;
{
    l_interval y;
    int stagsave = stagprec,
	stagmax = 39;
    if (!CXSC_Sqrt3_initialized)
    {
	std::string str;
	std::cout << SaveOpt;
	std::cout << Hex;
	str = "+1BB67AE8584CAAe7FC";
	str >> CXSC_Sqrt3[0];
	str = "+1CEC95D0B5C1E3e7C6";
	str >> CXSC_Sqrt3[1];
	str = "-1F11DB689F2CCFe78E";
	str >> CXSC_Sqrt3[2];
	str = "+13DA4798C720A6e758";
	str >> CXSC_Sqrt3[3];
	str = "+121B9169B89243e722";
	str >> CXSC_Sqrt3[4];
	str = "-1813508751212Be6E9";
	str >> CXSC_Sqrt3[5];
	str = "-1B3D547B775C1Ee6B2";
	str >> CXSC_Sqrt3[6];
	str = "-19D986D92E2F0Ae679";
	str >> CXSC_Sqrt3[7];
	str = "+1A34334CE806B6e642";
	str >> CXSC_Sqrt3[8];
	str = "+1A383B9E122E61e60C";
	str >> CXSC_Sqrt3[9];
	str = "+1C61D736F2F6F2e5D5";
	str >> CXSC_Sqrt3[10];
	str = "-10AF49233F9250e59E";
	str >> CXSC_Sqrt3[11];
	str = "-1558A109EC0523e567";
	str >> CXSC_Sqrt3[12];
	str = "+1F799D4D4FF2BCe531";
	str >> CXSC_Sqrt3[13];
	str = "-1AD7B219E34EDBe4FB";
	str >> CXSC_Sqrt3[14];
	str = "+15AB940B6677E3e4C5";
	str >> CXSC_Sqrt3[15];
	str = "-1D9B2A8203B8F0e48E";
	str >> CXSC_Sqrt3[16];
	str = "-1DB0C8975A3834e458";
	str >> CXSC_Sqrt3[17];
	str = "-1BCAAB3F6BE884e422";
	str >> CXSC_Sqrt3[18];
	str = "+14C70ADB1EC1BBe3E8";
	str >> CXSC_Sqrt3[19];
	str = "-14E1EF77987E55e3AF";
	str >> CXSC_Sqrt3[20];
	str = "-19695FC6269D28e378";
	str >> CXSC_Sqrt3[21];
	str = "+10D0652AAC5936e342";
	str >> CXSC_Sqrt3[22];
	str = "-1BD0891D370824e30C";
	str >> CXSC_Sqrt3[23];
	str = "-129B4C6252D061e2D4";
	str >> CXSC_Sqrt3[24];
	str = "+1DC9B1A4C31275e29E";
	str >> CXSC_Sqrt3[25];
	str = "+11FF9B8422294Ee267";
	str >> CXSC_Sqrt3[26];
	str = "-1E4A6AA47F3A85e231";
	str >> CXSC_Sqrt3[27];
	str = "+17043E01AA3F3De1FA";
	str >> CXSC_Sqrt3[28];
	str = "+188EF377D2D5B6e1C0";
	str >> CXSC_Sqrt3[29];
	str = "-1735E8C815F031e185";
	str >> CXSC_Sqrt3[30];
	str = "-1B89330FD8417Ce14F";
	str >> CXSC_Sqrt3[31];
	str = "+16D1A627670F5Ce117";
	str >> CXSC_Sqrt3[32];
	str = "+1AF43BBA8154D3e0DB";
	str >> CXSC_Sqrt3[33];
	str = "+1DA9A969A91295e0A5";
	str >> CXSC_Sqrt3[34];
	str = "-1636594394C675e06E";
	str >> CXSC_Sqrt3[35];
	str = "+1064B9DA1A3185e037";
	str >> CXSC_Sqrt3[36];
	str = "-1CE514CF1825CCe001";
	str >> CXSC_Sqrt3[37];
	str = "+10000000000000e000";
	str >> CXSC_Sqrt3[38];
	str = "+10000000000001e000";
	str >> CXSC_Sqrt3[39];

	CXSC_Sqrt3_initialized = true;
	std::cout << RestoreOpt;
    } 
    stagprec = stagmax;
    y = adjust(l_interval(0));

    for (int i=0; i<=stagmax; i++)
	y[i+1] = CXSC_Sqrt3[i];

    stagprec = stagsave;
    y = adjust(y);

    return lx_interval(-1021,y);
} // sqrt(3)

static real CXSC_Ln2r[40]; // CXSC_Ln2r[0], ... ,CXSC_Ln2r[39]
static bool CXSC_Ln2r_initialized = false;

lx_interval Ln2r_lx_interval() throw()
// Inclusion of 1/ln(2), Blomquist, 15.06.2007;
{
    l_interval y;
    int stagsave = stagprec,
	stagmax = 39;
    if (!CXSC_Ln2r_initialized)
    {
	std::string str;
	std::cout << SaveOpt;
	std::cout << Hex;
	str = "+171547652B82FEe7FC";
	str >> CXSC_Ln2r[0];
	str = "+1777D0FFDA0D24e7C4";
	str >> CXSC_Ln2r[1];
	str = "-160BB8A5442AB9e78E";
	str >> CXSC_Ln2r[2];
	str = "-14B52D3BA6D74De756";
	str >> CXSC_Ln2r[3];
	str = "+19A342648FBC39e720";
	str >> CXSC_Ln2r[4];
	str = "-1E0455744994EEe6EA";
	str >> CXSC_Ln2r[5];
	str = "+1B25EEB82D7C16e6B4";
	str >> CXSC_Ln2r[6];
	str = "+1F5485CF306255e67E";
	str >> CXSC_Ln2r[7];
	str = "-1EC07680A1F958e648";
	str >> CXSC_Ln2r[8];
	str = "-106326680EB5B6e612";
	str >> CXSC_Ln2r[9];
	str = "-1B3D04C549BC98e5DC";
	str >> CXSC_Ln2r[10];
	str = "+1EABCEAD10305Be5A6";
	str >> CXSC_Ln2r[11];
	str = "-14440C57D7AB97e56D";
	str >> CXSC_Ln2r[12];
	str = "-17185D42A4E6D6e536";
	str >> CXSC_Ln2r[13];
	str = "-1F332B5BE48526e4FE";
	str >> CXSC_Ln2r[14];
	str = "+12CE4F199E108De4C8";
	str >> CXSC_Ln2r[15];
	str = "-18DAFCC6077F2Ae48F";
	str >> CXSC_Ln2r[16];
	str = "+19ABB71EC25E12e458";
	str >> CXSC_Ln2r[17];
	str = "-11473D7A3366BDe41F";
	str >> CXSC_Ln2r[18];
	str = "-125DF4E28B5ED4e3E8";
	str >> CXSC_Ln2r[19];
	str = "+1C64262D010330e3B2";
	str >> CXSC_Ln2r[20];
	str = "-17DCAE42742BDEe37C";
	str >> CXSC_Ln2r[21];
	str = "+109C8C7E7B896Fe346";
	str >> CXSC_Ln2r[22];
	str = "+10C470FE2464B9e310";
	str >> CXSC_Ln2r[23];
	str = "-1B5F6CFB7C34BEe2DA";
	str >> CXSC_Ln2r[24];
	str = "-125E5DBA4A1165e2A1";
	str >> CXSC_Ln2r[25];
	str = "-1FA683975309E6e26A";
	str >> CXSC_Ln2r[26];
	str = "-140C23C4E5CC64e233";
	str >> CXSC_Ln2r[27];
	str = "+117670EC70E797e1FD";
	str >> CXSC_Ln2r[28];
	str = "+1B2A04B8E7416Ce1C7";
	str >> CXSC_Ln2r[29];
	str = "+11D96159397087e18F";
	str >> CXSC_Ln2r[30];
	str = "+10E29D810B4C60e159";
	str >> CXSC_Ln2r[31];
	str = "+1D7442ECEFEFA1e123";
	str >> CXSC_Ln2r[32];
	str = "+1CE25B70026529e0ED";
	str >> CXSC_Ln2r[33];
	str = "-12CA24549E0811e0B7";
	str >> CXSC_Ln2r[34];
	str = "+1220755E0827AEe080";
	str >> CXSC_Ln2r[35];
	str = "+1086BCE30D4370e04A";
	str >> CXSC_Ln2r[36];
	str = "-16FF855E4293BCe011";
	str >> CXSC_Ln2r[37];
	str = "+10000000002A50e000";
	str >> CXSC_Ln2r[38];
	str = "+10000000002A51e000";
	str >> CXSC_Ln2r[39];

	CXSC_Ln2r_initialized = true;
	std::cout << RestoreOpt;
    } 
    stagprec = stagmax;
    y = adjust(l_interval(0));

    for (int i=0; i<=stagmax; i++)
	y[i+1] = CXSC_Ln2r[i];

    stagprec = stagsave;
    y = adjust(y);

    return lx_interval(-1021,y);
} // 1/ln(2)

static real CXSC_Pid3[40]; // CXSC_Pid3[0], ... ,CXSC_Pid3[39]
static bool CXSC_Pid3_initialized = false;

lx_interval Pid3_lx_interval() throw()
// Inclusion of Pi/3, Blomquist, 15.06.2007;
{
    l_interval y;
    int stagsave = stagprec,
	stagmax = 39;
    if (!CXSC_Pid3_initialized)
    {
	std::string str;
	std::cout << SaveOpt;
	std::cout << Hex;
	str = "+10C152382D7366e7FD";
	str >> CXSC_Pid3[0];
	str = "-1EE6913347C2A6e7C7";
	str >> CXSC_Pid3[1];
	str = "-14BBA47A9E5FD2e78F";
	str >> CXSC_Pid3[2];
	str = "-1CCAEF65529B02e759";
	str >> CXSC_Pid3[3];
	str = "+197CB7BCC18B87e722";
	str >> CXSC_Pid3[4];
	str = "-13EBBDA1FF3058e6EC";
	str >> CXSC_Pid3[5];
	str = "-11D10CB320F4D1e6B4";
	str >> CXSC_Pid3[6];
	str = "+1958EB892987ECe67D";
	str >> CXSC_Pid3[7];
	str = "+167C54B11CF247e647";
	str >> CXSC_Pid3[8];
	str = "+12C2E985923A44e60E";
	str >> CXSC_Pid3[9];
	str = "+1945484A2DD81Fe5D6";
	str >> CXSC_Pid3[10];
	str = "+1197A9E475D54Fe59E";
	str >> CXSC_Pid3[11];
	str = "-1E181FEE158585e568";
	str >> CXSC_Pid3[12];
	str = "+1047FCE7066A6Ee532";
	str >> CXSC_Pid3[13];
	str = "+1D1A8602EA0C85e4FC";
	str >> CXSC_Pid3[14];
	str = "+14430C5998BF34e4C6";
	str >> CXSC_Pid3[15];
	str = "+173BF40AAD43D9e48F";
	str >> CXSC_Pid3[16];
	str = "-137B014DDEDCF5e459";
	str >> CXSC_Pid3[17];
	str = "-1A5F1B210EE7C5e420";
	str >> CXSC_Pid3[18];
	str = "+151B536DDF9502e3EA";
	str >> CXSC_Pid3[19];
	str = "+10E4DB4F709CEEe3B4";
	str >> CXSC_Pid3[20];
	str = "+16841F78EC058Ee37E";
	str >> CXSC_Pid3[21];
	str = "+1D269E370AFA06e346";
	str >> CXSC_Pid3[22];
	str = "+119123BD75E37Be310";
	str >> CXSC_Pid3[23];
	str = "+1C7DBAADF64D9De2DA";
	str >> CXSC_Pid3[24];
	str = "+16CC595AEA086De2A4";
	str >> CXSC_Pid3[25];
	str = "+1942EC979DED29e26E";
	str >> CXSC_Pid3[26];
	str = "+1EFBE875957C10e238";
	str >> CXSC_Pid3[27];
	str = "+133B7D68BA4029e1FF";
	str >> CXSC_Pid3[28];
	str = "-11EB0DA382BF6Ce1C9";
	str >> CXSC_Pid3[29];
	str = "+1970EDF4B943FDe193";
	str >> CXSC_Pid3[30];
	str = "-11C6A6D14BBC74e15C";
	str >> CXSC_Pid3[31];
	str = "+1FBE371E3DC1D2e125";
	str >> CXSC_Pid3[32];
	str = "-1F34D225753A55e0EF";
	str >> CXSC_Pid3[33];
	str = "+1D0DA1E2E38EC1e0B7";
	str >> CXSC_Pid3[34];
	str = "-18C889B4CA7CA6e07E";
	str >> CXSC_Pid3[35];
	str = "+1B346B8DAF1FA8e048";
	str >> CXSC_Pid3[36];
	str = "+1326EDF35258AEe012";
	str >> CXSC_Pid3[37];
	str = "-1000000000DEDBe000";
	str >> CXSC_Pid3[38];
	str = "-1000000000DEDAe000";
	str >> CXSC_Pid3[39];

	CXSC_Pid3_initialized = true;
	std::cout << RestoreOpt;
    } 
    stagprec = stagmax;
    y = adjust(l_interval(0));

    for (int i=0; i<=stagmax; i++)
	y[i+1] = CXSC_Pid3[i];

    stagprec = stagsave;
    y = adjust(y);

    return lx_interval(-1022,y);
} // Pi/3

static real CXSC_SqrtPir[40]; // CXSC_SqrtPir[0], ... ,CXSC_SqrtPir[39]
static bool CXSC_SqrtPir_initialized = false;

lx_interval SqrtPir_lx_interval() throw()
// Inclusion of 1/sqrt(Pi), Blomquist, 15.06.2007;
{
    l_interval y;
    int stagsave = stagprec,
	stagmax = 39;
    if (!CXSC_SqrtPir_initialized)
    {
	std::string str;
	std::cout << SaveOpt;
	std::cout << Hex;
	str = "+120DD750429B6De7FD";
	str >> CXSC_SqrtPir[0];
	str = "+11AE3A914FED80e7C5";
	str >> CXSC_SqrtPir[1];
	str = "-13CBBEBF65F145e78E";
	str >> CXSC_SqrtPir[2];
	str = "-1E0C574632F53Ee757";
	str >> CXSC_SqrtPir[3];
	str = "-1E6633BE9E7F15e721";
	str >> CXSC_SqrtPir[4];
	str = "+1CF859270F1141e6EA";
	str >> CXSC_SqrtPir[5];
	str = "-1FE4FB499C328Ae6B3";
	str >> CXSC_SqrtPir[6];
	str = "-10B82C446DC78De67C";
	str >> CXSC_SqrtPir[7];
	str = "-1878B089078800e646";
	str >> CXSC_SqrtPir[8];
	str = "-13DAEADA9E233Ee60E";
	str >> CXSC_SqrtPir[9];
	str = "+1137197A708BD2e5D8";
	str >> CXSC_SqrtPir[10];
	str = "-109009506D5BA2e59D";
	str >> CXSC_SqrtPir[11];
	str = "+17C9F0B5951E94e567";
	str >> CXSC_SqrtPir[12];
	str = "-1735F4949633A4e530";
	str >> CXSC_SqrtPir[13];
	str = "-146014DBC90D0Ee4FA";
	str >> CXSC_SqrtPir[14];
	str = "+1CAB0B222EEEA0e4C4";
	str >> CXSC_SqrtPir[15];
	str = "+1B1C750754B40Ae48E";
	str >> CXSC_SqrtPir[16];
	str = "-16B2CD2E72C16Ee456";
	str >> CXSC_SqrtPir[17];
	str = "-148C024FF194B2e420";
	str >> CXSC_SqrtPir[18];
	str = "+1CF866DD09628De3EA";
	str >> CXSC_SqrtPir[19];
	str = "-16CBF3DC0C536Ee3B3";
	str >> CXSC_SqrtPir[20];
	str = "+1EEDA3436CC85Fe37C";
	str >> CXSC_SqrtPir[21];
	str = "+16928D2C52986Ae346";
	str >> CXSC_SqrtPir[22];
	str = "+1EF82FD6BDE19De30E";
	str >> CXSC_SqrtPir[23];
	str = "+1218F360779AEEe2D8";
	str >> CXSC_SqrtPir[24];
	str = "+14F13DB14D3AC2e2A2";
	str >> CXSC_SqrtPir[25];
	str = "+1461BD3C8DC495e26C";
	str >> CXSC_SqrtPir[26];
	str = "-1CAE7855A13FF8e234";
	str >> CXSC_SqrtPir[27];
	str = "-18CEA8571A02F7e1FE";
	str >> CXSC_SqrtPir[28];
	str = "-1AC80A19057BBDe1C8";
	str >> CXSC_SqrtPir[29];
	str = "+1A1910D82DC198e191";
	str >> CXSC_SqrtPir[30];
	str = "+16B8F9198BC17Fe15B";
	str >> CXSC_SqrtPir[31];
	str = "+1D7B579C6CAF5De123";
	str >> CXSC_SqrtPir[32];
	str = "+1E97B95E80FE25e0ED";
	str >> CXSC_SqrtPir[33];
	str = "-188EF7630D4F86e0AF";
	str >> CXSC_SqrtPir[34];
	str = "+1988FBA498490Ae076";
	str >> CXSC_SqrtPir[35];
	str = "+178235EAEC9403e03F";
	str >> CXSC_SqrtPir[36];
	str = "+1067F03DFDFF93e005";
	str >> CXSC_SqrtPir[37];
	str = "+10000000000006e000";
	str >> CXSC_SqrtPir[38];
	str = "+10000000000007e000";
	str >> CXSC_SqrtPir[39];

	CXSC_SqrtPir_initialized = true;
	std::cout << RestoreOpt;
    } 
    stagprec = stagmax;
    y = adjust(l_interval(0));

    for (int i=0; i<=stagmax; i++)
	y[i+1] = CXSC_SqrtPir[i];

    stagprec = stagsave;
    y = adjust(y);

    return lx_interval(-1023,y);
} // 1/sqrt(Pi)

static real CXSC_Sqrt2Pir[40]; // CXSC_Sqrt2Pir[0], ... ,CXSC_Sqrt2Pir[39]
static bool CXSC_Sqrt2Pir_initialized = false;

lx_interval Sqrt2Pir_lx_interval() throw()
// Inclusion of 1/sqrt(2*Pi), Blomquist, 15.06.2007;
{
    l_interval y;
    int stagsave = stagprec,
	stagmax = 39;
    if (!CXSC_Sqrt2Pir_initialized)
    {
	std::string str;
	std::cout << SaveOpt;
	std::cout << Hex;
	str = "+19884533D43651e7FC";
	str >> CXSC_Sqrt2Pir[0];
	str = "-1CBC0D30EBFD15e7C6";
	str >> CXSC_Sqrt2Pir[1];
	str = "-1C7402C7D60CFBe78E";
	str >> CXSC_Sqrt2Pir[2];
	str = "+12706D8C0471B5e756";
	str >> CXSC_Sqrt2Pir[3];
	str = "-1FF6718B45881De720";
	str >> CXSC_Sqrt2Pir[4];
	str = "-13AABB82C248DCe6EA";
	str >> CXSC_Sqrt2Pir[5];
	str = "-1458A899162EE4e6B1";
	str >> CXSC_Sqrt2Pir[6];
	str = "-14EBD8868F41EBe67A";
	str >> CXSC_Sqrt2Pir[7];
	str = "+13278E993445F1e642";
	str >> CXSC_Sqrt2Pir[8];
	str = "-1CC019F5F4780Ae60C";
	str >> CXSC_Sqrt2Pir[9];
	str = "+147CE4B4ECDBD7e5D6";
	str >> CXSC_Sqrt2Pir[10];
	str = "-19A3DCC6A3534Be59E";
	str >> CXSC_Sqrt2Pir[11];
	str = "+11379A7BA8CB0Ae568";
	str >> CXSC_Sqrt2Pir[12];
	str = "-12D909C875312Ee531";
	str >> CXSC_Sqrt2Pir[13];
	str = "+1C1CEC4882C77Be4FA";
	str >> CXSC_Sqrt2Pir[14];
	str = "-14C4078263DF36e4C4";
	str >> CXSC_Sqrt2Pir[15];
	str = "+1AB3FC8D2AB243e48E";
	str >> CXSC_Sqrt2Pir[16];
	str = "+17B9172454310Ae458";
	str >> CXSC_Sqrt2Pir[17];
	str = "-1444B6B781B7F2e422";
	str >> CXSC_Sqrt2Pir[18];
	str = "-1DB5C6773B74B7e3EC";
	str >> CXSC_Sqrt2Pir[19];
	str = "-12D4CD4FAB8CF9e3B5";
	str >> CXSC_Sqrt2Pir[20];
	str = "-12C319ACF346DCe37D";
	str >> CXSC_Sqrt2Pir[21];
	str = "+193ED298857CB8e346";
	str >> CXSC_Sqrt2Pir[22];
	str = "+1AB87659565E92e30E";
	str >> CXSC_Sqrt2Pir[23];
	str = "-1AEB785019F78Ee2D6";
	str >> CXSC_Sqrt2Pir[24];
	str = "-17DAF38DE68CA0e29F";
	str >> CXSC_Sqrt2Pir[25];
	str = "-14D672D025580Ce265";
	str >> CXSC_Sqrt2Pir[26];
	str = "-17AA87F2ABB794e22F";
	str >> CXSC_Sqrt2Pir[27];
	str = "+16E85953CBD917e1F9";
	str >> CXSC_Sqrt2Pir[28];
	str = "-10AB555C9A9735e1C3";
	str >> CXSC_Sqrt2Pir[29];
	str = "+1020FEB8ED1EA2e18A";
	str >> CXSC_Sqrt2Pir[30];
	str = "+18282C79079F71e152";
	str >> CXSC_Sqrt2Pir[31];
	str = "+13D282FF699FC4e11C";
	str >> CXSC_Sqrt2Pir[32];
	str = "-1A547E139AE10Ce0E6";
	str >> CXSC_Sqrt2Pir[33];
	str = "+167FCF0E311B0De0AE";
	str >> CXSC_Sqrt2Pir[34];
	str = "+1AC5D5E32ED719e078";
	str >> CXSC_Sqrt2Pir[35];
	str = "+1B311EB9071956e042";
	str >> CXSC_Sqrt2Pir[36];
	str = "+1D237BE89A6494e00C";
	str >> CXSC_Sqrt2Pir[37];
	str = "+10000000000075e000";
	str >> CXSC_Sqrt2Pir[38];
	str = "+10000000000076e000";
	str >> CXSC_Sqrt2Pir[39];

	CXSC_Sqrt2Pir_initialized = true;
	std::cout << RestoreOpt;
    } 
    stagprec = stagmax;
    y = adjust(l_interval(0));

    for (int i=0; i<=stagmax; i++)
	y[i+1] = CXSC_Sqrt2Pir[i];

    stagprec = stagsave;
    y = adjust(y);

    return lx_interval(-1023,y);
} // 1/sqrt(2*Pi)

static real CXSC_LnPi[40]; // CXSC_LnPi[0], ... ,CXSC_LnPi[39]
static bool CXSC_LnPi_initialized = false;

lx_interval LnPi_lx_interval() throw()
// Inclusion of ln(Pi), Blomquist, 15.06.2007;
{
    l_interval y;
    int stagsave = stagprec,
	stagmax = 39;
    if (!CXSC_LnPi_initialized)
    {
	std::string str;
	std::cout << SaveOpt;
	std::cout << Hex;
	str = "+1250D048E7A1BDe7FD";
	str >> CXSC_LnPi[0];
	str = "+17ABF2AD8D5088e7C4";
	str >> CXSC_LnPi[1];
	str = "-16CCF43244818Ae78C";
	str >> CXSC_LnPi[2];
	str = "+1F9303719C0176e756";
	str >> CXSC_LnPi[3];
	str = "+15DF52611CB54Ee720";
	str >> CXSC_LnPi[4];
	str = "-1D9056E74F8C97e6EA";
	str >> CXSC_LnPi[5];
	str = "+100B095B6C2E1Ae6B3";
	str >> CXSC_LnPi[6];
	str = "-18C7557878A9E7e67D";
	str >> CXSC_LnPi[7];
	str = "+1B9BBBB4F4CEE7e646";
	str >> CXSC_LnPi[8];
	str = "+1B477FCC702F86e610";
	str >> CXSC_LnPi[9];
	str = "+141F1344A31799e5DA";
	str >> CXSC_LnPi[10];
	str = "+1B6740BE95CD58e5A4";
	str >> CXSC_LnPi[11];
	str = "-1F2C63904D27DBe56C";
	str >> CXSC_LnPi[12];
	str = "+1426F00B933976e534";
	str >> CXSC_LnPi[13];
	str = "+125703BE5FAA20e4FE";
	str >> CXSC_LnPi[14];
	str = "-1DADAE5397F95Be4C7";
	str >> CXSC_LnPi[15];
	str = "+17C9D110381543e48F";
	str >> CXSC_LnPi[16];
	str = "-1259230E627FCAe459";
	str >> CXSC_LnPi[17];
	str = "+191CEAB6B13A33e422";
	str >> CXSC_LnPi[18];
	str = "+109D49A13CB595e3EB";
	str >> CXSC_LnPi[19];
	str = "-12C574CDCD41C2e3B4";
	str >> CXSC_LnPi[20];
	str = "+1D4141476C3E9De37E";
	str >> CXSC_LnPi[21];
	str = "+1D26D892B64467e344";
	str >> CXSC_LnPi[22];
	str = "-16BAF3B607E1ADe30B";
	str >> CXSC_LnPi[23];
	str = "+165A768D8BC5ADe2D5";
	str >> CXSC_LnPi[24];
	str = "+12185364B32BD1e29D";
	str >> CXSC_LnPi[25];
	str = "-114D72550F0B90e266";
	str >> CXSC_LnPi[26];
	str = "+1E586BAEEB8BF4e230";
	str >> CXSC_LnPi[27];
	str = "-1F4B9322D4506Fe1F8";
	str >> CXSC_LnPi[28];
	str = "+16D32BAA9A4FCCe1C2";
	str >> CXSC_LnPi[29];
	str = "+1A12D8CF8CC6DAe18B";
	str >> CXSC_LnPi[30];
	str = "-1215CF3BD682CAe155";
	str >> CXSC_LnPi[31];
	str = "-184CA7D8873E45e11D";
	str >> CXSC_LnPi[32];
	str = "-1F02ECC3E58C6Ee0E7";
	str >> CXSC_LnPi[33];
	str = "-11F6EC8ED0D92Be0B1";
	str >> CXSC_LnPi[34];
	str = "-199F29ACE1FC18e077";
	str >> CXSC_LnPi[35];
	str = "+119F3673AA919Ae041";
	str >> CXSC_LnPi[36];
	str = "-1A8359A2831626e00A";
	str >> CXSC_LnPi[37];
	str = "-10000000000072e000";
	str >> CXSC_LnPi[38];
	str = "-10000000000071e000";
	str >> CXSC_LnPi[39];

	CXSC_LnPi_initialized = true;
	std::cout << RestoreOpt;
    } 
    stagprec = stagmax;
    y = adjust(l_interval(0));

    for (int i=0; i<=stagmax; i++)
	y[i+1] = CXSC_LnPi[i];

    stagprec = stagsave;
    y = adjust(y);

    return lx_interval(-1022,y);
} // ln(Pi)

static real CXSC_Ln2Pi[40]; // CXSC_Ln2Pi[0], ... ,CXSC_Ln2Pi[39]
static bool CXSC_Ln2Pi_initialized = false;

lx_interval Ln2Pi_lx_interval() throw()
// Inclusion of ln(2*Pi), Blomquist, 15.06.2007;
{
    l_interval y;
    int stagsave = stagprec,
	stagmax = 39;
    if (!CXSC_Ln2Pi_initialized)
    {
	std::string str;
	std::cout << SaveOpt;
	std::cout << Hex;
	str = "+1D67F1C864BEB5e7FC";
	str >> CXSC_Ln2Pi[0];
	str = "-165B5A1B7FF5DFe7C6";
	str >> CXSC_Ln2Pi[1];
	str = "-1B7F70C13DC1CCe78F";
	str >> CXSC_Ln2Pi[2];
	str = "+13458B4DDEC6A3e759";
	str >> CXSC_Ln2Pi[3];
	str = "+133DAA155D2130e721";
	str >> CXSC_Ln2Pi[4];
	str = "-18A007FC5E501Be6EB";
	str >> CXSC_Ln2Pi[5];
	str = "-15406FA3AA9644e6B1";
	str >> CXSC_Ln2Pi[6];
	str = "-13E8D52A392CC9e67B";
	str >> CXSC_Ln2Pi[7];
	str = "-1A43099131E88De645";
	str >> CXSC_Ln2Pi[8];
	str = "-114835B6623C4De60F";
	str >> CXSC_Ln2Pi[9];
	str = "-1ABB7858CF827Ae5D9";
	str >> CXSC_Ln2Pi[10];
	str = "+1D8D7045A5A495e5A3";
	str >> CXSC_Ln2Pi[11];
	str = "+1A26094B3F6FC5e56C";
	str >> CXSC_Ln2Pi[12];
	str = "-1EF27932D0E3D0e534";
	str >> CXSC_Ln2Pi[13];
	str = "-12128804136AB6e4FD";
	str >> CXSC_Ln2Pi[14];
	str = "+15F8A4AC0BEE17e4C4";
	str >> CXSC_Ln2Pi[15];
	str = "+1892F2A5B69B5Fe48E";
	str >> CXSC_Ln2Pi[16];
	str = "+1CC7C09477ADCEe458";
	str >> CXSC_Ln2Pi[17];
	str = "-116DD579AF074Ae41F";
	str >> CXSC_Ln2Pi[18];
	str = "+190E43C1DCCD69e3E7";
	str >> CXSC_Ln2Pi[19];
	str = "-11F55BBD0978D3e3AF";
	str >> CXSC_Ln2Pi[20];
	str = "+167EC65B83F29Be378";
	str >> CXSC_Ln2Pi[21];
	str = "-14C0D466FC8C7Ae33C";
	str >> CXSC_Ln2Pi[22];
	str = "-1D56DE4860435Ce306";
	str >> CXSC_Ln2Pi[23];
	str = "-10C7B15DFFBDFCe2D0";
	str >> CXSC_Ln2Pi[24];
	str = "-15007E40803B52e299";
	str >> CXSC_Ln2Pi[25];
	str = "+1DF2A457B56D15e261";
	str >> CXSC_Ln2Pi[26];
	str = "-16B7CAD686151De22B";
	str >> CXSC_Ln2Pi[27];
	str = "-11F972F1A61CA1e1F5";
	str >> CXSC_Ln2Pi[28];
	str = "+1443CF52FBF6B4e1BE";
	str >> CXSC_Ln2Pi[29];
	str = "-12652AE82DC678e187";
	str >> CXSC_Ln2Pi[30];
	str = "-11712858901127e151";
	str >> CXSC_Ln2Pi[31];
	str = "+124D51F4842F1Fe11B";
	str >> CXSC_Ln2Pi[32];
	str = "+1B8F6B0823A92Ae0E4";
	str >> CXSC_Ln2Pi[33];
	str = "-1742244E0D8F40e0AB";
	str >> CXSC_Ln2Pi[34];
	str = "+127A8F1E2AEAD3e074";
	str >> CXSC_Ln2Pi[35];
	str = "-180BC6B9E8F00Ce03D";
	str >> CXSC_Ln2Pi[36];
	str = "+190933A24F0ECEe007";
	str >> CXSC_Ln2Pi[37];
	str = "-10000000000007e000";
	str >> CXSC_Ln2Pi[38];
	str = "-10000000000006e000";
	str >> CXSC_Ln2Pi[39];

	CXSC_Ln2Pi_initialized = true;
	std::cout << RestoreOpt;
    } 
    stagprec = stagmax;
    y = adjust(l_interval(0));

    for (int i=0; i<=stagmax; i++)
	y[i+1] = CXSC_Ln2Pi[i];

    stagprec = stagsave;
    y = adjust(y);

    return lx_interval(-1021,y);
} // ln(2*Pi)

static real CXSC_E[40]; // CXSC_E[0], ... ,CXSC_E[39]
static bool CXSC_E_initialized = false;

lx_interval E_lx_interval() throw()
// Inclusion of e, Blomquist, 15.06.2007;
{
    l_interval y;
    int stagsave = stagprec,
	stagmax = 39;
    if (!CXSC_E_initialized)
    {
	std::string str;
	std::cout << SaveOpt;
	std::cout << Hex;
	str = "+15BF0A8B145769e7FD";
	str >> CXSC_E[0];
	str = "+14D57EE2B1013Ae7C7";
	str >> CXSC_E[1];
	str = "-1618713A31D3E2e78F";
	str >> CXSC_E[2];
	str = "+1C5A6D2B53C26De759";
	str >> CXSC_E[3];
	str = "-1F75CDE60219B6e723";
	str >> CXSC_E[4];
	str = "-188C76D93041A1e6EC";
	str >> CXSC_E[5];
	str = "+12FE363630C75Ee6B6";
	str >> CXSC_E[6];
	str = "-1C25F937F544EEe680";
	str >> CXSC_E[7];
	str = "-1E852C20E12A2Ae64A";
	str >> CXSC_E[8];
	str = "-14D4F6DE605705e60F";
	str >> CXSC_E[9];
	str = "-1F3225EF539355e5D5";
	str >> CXSC_E[10];
	str = "-16109728625547e59F";
	str >> CXSC_E[11];
	str = "-194301506D94CFe569";
	str >> CXSC_E[12];
	str = "-1879C78F8CBA44e533";
	str >> CXSC_E[13];
	str = "-1D5976250C1018e4FA";
	str >> CXSC_E[14];
	str = "+1C877C56284DABe4C4";
	str >> CXSC_E[15];
	str = "+1E73530ACCA4F5e48E";
	str >> CXSC_E[16];
	str = "-1F161A150FD53Ae458";
	str >> CXSC_E[17];
	str = "+159927DB0E8845e41F";
	str >> CXSC_E[18];
	str = "+12976591C7F773e3E9";
	str >> CXSC_E[19];
	str = "-1525489F280B98e3B2";
	str >> CXSC_E[20];
	str = "+1D4F42A3DE394Ee37A";
	str >> CXSC_E[21];
	str = "-16A3522431391Be341";
	str >> CXSC_E[22];
	str = "+1D8C8583D3E477e30B";
	str >> CXSC_E[23];
	str = "+14DAE13C05F9C4e2D1";
	str >> CXSC_E[24];
	str = "-19040E899FE5FEe29B";
	str >> CXSC_E[25];
	str = "+19A50685EC322Ee265";
	str >> CXSC_E[26];
	str = "+17F4E74C2C1FFCe22F";
	str >> CXSC_E[27];
	str = "+1C9E2465DDE503e1F9";
	str >> CXSC_E[28];
	str = "+1E1FF1D8DA637De1BF";
	str >> CXSC_E[29];
	str = "+1AE6776BF9785Ee189";
	str >> CXSC_E[30];
	str = "-1EEFFD1D38873Ee153";
	str >> CXSC_E[31];
	str = "-105D2F89A72197e11D";
	str >> CXSC_E[32];
	str = "+11360D977FD443e0E7";
	str >> CXSC_E[33];
	str = "+168470C23F9FBAe0B1";
	str >> CXSC_E[34];
	str = "-10E552624D737Ee07B";
	str >> CXSC_E[35];
	str = "-148879616420CCe045";
	str >> CXSC_E[36];
	str = "-1FEE3CF25C81B1e00F";
	str >> CXSC_E[37];
	str = "-10000000001233e000";
	str >> CXSC_E[38];
	str = "-10000000001232e000";
	str >> CXSC_E[39];

	CXSC_E_initialized = true;
	std::cout << RestoreOpt;
    } 
    stagprec = stagmax;
    y = adjust(l_interval(0));

    for (int i=0; i<=stagmax; i++)
	y[i+1] = CXSC_E[i];

    stagprec = stagsave;
    y = adjust(y);

    return lx_interval(-1021,y);
} // e

static real CXSC_EpPi[40]; // CXSC_EpPi[0], ... ,CXSC_EpPi[39]
static bool CXSC_EpPi_initialized = false;

lx_interval EpPi_lx_interval() throw()
// Inclusion of e^Pi, Blomquist, 15.06.2007;
{
    l_interval y;
    int stagsave = stagprec,
	stagmax = 39;
    if (!CXSC_EpPi_initialized)
    {
	std::string str;
	std::cout << SaveOpt;
	std::cout << Hex;
	str = "+1724046EB0933Ae7FD";
	str >> CXSC_EpPi[0];
	str = "-184C962DD81952e7C7";
	str >> CXSC_EpPi[1];
	str = "-12D659C0BCD22Ee790";
	str >> CXSC_EpPi[2];
	str = "+117496B8A92F91e75A";
	str >> CXSC_EpPi[3];
	str = "+16A8C4203E5FCDe724";
	str >> CXSC_EpPi[4];
	str = "-166B11F99A663Be6EE";
	str >> CXSC_EpPi[5];
	str = "-118EC2076DABB1e6B8";
	str >> CXSC_EpPi[6];
	str = "+19776E5BEB18A5e682";
	str >> CXSC_EpPi[7];
	str = "+1AD4091E84B051e64C";
	str >> CXSC_EpPi[8];
	str = "+1E89AA12909B40e616";
	str >> CXSC_EpPi[9];
	str = "+1ACE3C0DDBB994e5DD";
	str >> CXSC_EpPi[10];
	str = "+141EC9379CBBFEe5A6";
	str >> CXSC_EpPi[11];
	str = "+1FC4E78D00A016e56D";
	str >> CXSC_EpPi[12];
	str = "+1608BE35B9A409e537";
	str >> CXSC_EpPi[13];
	str = "-1A0D8AA90EB6B9e4FD";
	str >> CXSC_EpPi[14];
	str = "+106FE8AFD21ACFe4C7";
	str >> CXSC_EpPi[15];
	str = "+1C072FEA1BFCAFe48F";
	str >> CXSC_EpPi[16];
	str = "+1915B9F352EC68e455";
	str >> CXSC_EpPi[17];
	str = "-13FA07C37897E9e41E";
	str >> CXSC_EpPi[18];
	str = "-1EC01C89B8DDFFe3E8";
	str >> CXSC_EpPi[19];
	str = "+1EDC3A402336AFe3B0";
	str >> CXSC_EpPi[20];
	str = "-12677080620EA5e378";
	str >> CXSC_EpPi[21];
	str = "-1C63FD21D891DEe340";
	str >> CXSC_EpPi[22];
	str = "-1FB6165FFF8730e309";
	str >> CXSC_EpPi[23];
	str = "-177AB93E2523EFe2D3";
	str >> CXSC_EpPi[24];
	str = "+16D78E0B522E2Ce29C";
	str >> CXSC_EpPi[25];
	str = "-17473D7DD61EBEe266";
	str >> CXSC_EpPi[26];
	str = "-1F082665C53E27e22A";
	str >> CXSC_EpPi[27];
	str = "-17CDF823ACB5D6e1F3";
	str >> CXSC_EpPi[28];
	str = "-1D95D856C4BF74e1BC";
	str >> CXSC_EpPi[29];
	str = "-1327665D26E23Ae186";
	str >> CXSC_EpPi[30];
	str = "-1EC5804BDCCA81e150";
	str >> CXSC_EpPi[31];
	str = "-1C73760E976CC3e117";
	str >> CXSC_EpPi[32];
	str = "-10B1DCE92BE86Ce0E0";
	str >> CXSC_EpPi[33];
	str = "-17372866D0A1CCe0AA";
	str >> CXSC_EpPi[34];
	str = "-15510B0AF58D1Ee074";
	str >> CXSC_EpPi[35];
	str = "+1B9820D80B02D9e03C";
	str >> CXSC_EpPi[36];
	str = "-17765315D853BAe002";
	str >> CXSC_EpPi[37];
	str = "-10000000000001e000";
	str >> CXSC_EpPi[38];
	str = "-10000000000000e000";
	str >> CXSC_EpPi[39];

	CXSC_EpPi_initialized = true;
	std::cout << RestoreOpt;
    } 
    stagprec = stagmax;
    y = adjust(l_interval(0));

    for (int i=0; i<=stagmax; i++)
	y[i+1] = CXSC_EpPi[i];

    stagprec = stagsave;
    y = adjust(y);

    return lx_interval(-1018,y);
} // e^Pi

static real CXSC_EulerGamma[40]; // CXSC_EulerGamma[0],...,CXSC_EulerGamma[39]
static bool CXSC_EulerGamma_initialized = false;

lx_interval EulerGamma_lx_interval() throw()
// Inclusion of EulerGamma, Blomquist, 15.06.2007;
{
    l_interval y;
    int stagsave = stagprec,
	stagmax = 39;
    if (!CXSC_EulerGamma_initialized)
    {
	std::string str;
	std::cout << SaveOpt;
	std::cout << Hex;
	str = "+12788CFC6FB619e7FD";
	str >> CXSC_EulerGamma[0];
	str = "-16CB90701FBFABe7C4";
	str >> CXSC_EulerGamma[1];
	str = "-134A95E3133C51e78E";
	str >> CXSC_EulerGamma[2];
	str = "+19730064300F7De758";
	str >> CXSC_EulerGamma[3];
	str = "-171ECA0084E369e721";
	str >> CXSC_EulerGamma[4];
	str = "-1302FE2B078898e6EB";
	str >> CXSC_EulerGamma[5];
	str = "+192732D88415F4e6B4";
	str >> CXSC_EulerGamma[6];
	str = "+11056AE9132136e67E";
	str >> CXSC_EulerGamma[7];
	str = "-17DC6F12E630A3e648";
	str >> CXSC_EulerGamma[8];
	str = "+175FD4B1BD70F2e611";
	str >> CXSC_EulerGamma[9];
	str = "-19BC9466120C20e5DB";
	str >> CXSC_EulerGamma[10];
	str = "-18FD5699260EADe5A5";
	str >> CXSC_EulerGamma[11];
	str = "-12EA987665551Fe56E";
	str >> CXSC_EulerGamma[12];
	str = "-1FB159BA4A423De537";
	str >> CXSC_EulerGamma[13];
	str = "+1FA543D43BCC60e501";
	str >> CXSC_EulerGamma[14];
	str = "-1E6F04E0F639F6e4C8";
	str >> CXSC_EulerGamma[15];
	str = "-1A23768654F43De490";
	str >> CXSC_EulerGamma[16];
	str = "-14F1C5CB4F55EBe457";
	str >> CXSC_EulerGamma[17];
	str = "+1E71DF52EDAA7Fe41F";
	str >> CXSC_EulerGamma[18];
	str = "+1C398F9B427E3Fe3E8";
	str >> CXSC_EulerGamma[19];
	str = "+1432C7402B3D24e3AF";
	str >> CXSC_EulerGamma[20];
	str = "-1810CF88C5F0D1e377";
	str >> CXSC_EulerGamma[21];
	str = "-1E9610AE5B38C5e340";
	str >> CXSC_EulerGamma[22];
	str = "+18220365594965e30A";
	str >> CXSC_EulerGamma[23];
	str = "-11F19DA40D550De2D4";
	str >> CXSC_EulerGamma[24];
	str = "+1936632B38107Ee29D";
	str >> CXSC_EulerGamma[25];
	str = "+11569F72C8E198e267";
	str >> CXSC_EulerGamma[26];
	str = "-1534E2EDC357EAe230";
	str >> CXSC_EulerGamma[27];
	str = "-126925B08D68FFe1F7";
	str >> CXSC_EulerGamma[28];
	str = "-1F62F86B1E6840e1C0";
	str >> CXSC_EulerGamma[29];
	str = "+133239C12DF171e188";
	str >> CXSC_EulerGamma[30];
	str = "+17E60989B189E2e14D";
	str >> CXSC_EulerGamma[31];
	str = "+1F6FB023E2AA98e117";
	str >> CXSC_EulerGamma[32];
	str = "+16911C6C6708F0e0DF";
	str >> CXSC_EulerGamma[33];
	str = "-1B7B61C9327A74e0A9";
	str >> CXSC_EulerGamma[34];
	str = "-16E397C5D924C3e073";
	str >> CXSC_EulerGamma[35];
	str = "-1435DBEDE3F382e03C";
	str >> CXSC_EulerGamma[36];
	str = "+1C0AEA1C4BB6FFe006";
	str >> CXSC_EulerGamma[37];
	str = "-1000000000000Ee000";
	str >> CXSC_EulerGamma[38];
	str = "-1000000000000De000";
	str >> CXSC_EulerGamma[39];

	CXSC_EulerGamma_initialized = true;
	std::cout << RestoreOpt;
    } 
    stagprec = stagmax;
    y = adjust(l_interval(0));

    for (int i=0; i<=stagmax; i++)
	y[i+1] = CXSC_EulerGamma[i];

    stagprec = stagsave;
    y = adjust(y);

    return lx_interval(-1023,y);
} // EulerGamma

static real CXSC_Catalan[40]; // CXSC_Catalan[0],...,CXSC_Catalan[39]
static bool CXSC_Catalan_initialized = false;

lx_interval Catalan_lx_interval() throw()
// Inclusion of Catalan, Blomquist, 15.06.2007;
{
    l_interval y;
    int stagsave = stagprec,
	stagmax = 39;
    if (!CXSC_Catalan_initialized)
    {
	std::string str;
	std::cout << SaveOpt;
	std::cout << Hex;
	str = "+1D4F9713E8135De7FC";
	str >> CXSC_Catalan[0];
	str = "+11485608B8DF4De7C3";
	str >> CXSC_Catalan[1];
	str = "-12F39C13BC1EC8e78D";
	str >> CXSC_Catalan[2];
	str = "+1C2FF8094A263Ee755";
	str >> CXSC_Catalan[3];
	str = "+168F335DBE5370e71F";
	str >> CXSC_Catalan[4];
	str = "+16291BBB16163Ee6E7";
	str >> CXSC_Catalan[5];
	str = "+124D663F739C43e6B1";
	str >> CXSC_Catalan[6];
	str = "-136A0725ED0E94e679";
	str >> CXSC_Catalan[7];
	str = "-1D3A26F9C06FCEe63E";
	str >> CXSC_Catalan[8];
	str = "-164E42486BFCD2e607";
	str >> CXSC_Catalan[9];
	str = "+14F358CFDEC843e5D1";
	str >> CXSC_Catalan[10];
	str = "-11EB82210976ABe59B";
	str >> CXSC_Catalan[11];
	str = "-17D31F6DF5E801e565";
	str >> CXSC_Catalan[12];
	str = "+13FD19CE3E396Ae52F";
	str >> CXSC_Catalan[13];
	str = "-1C8CBB3852FF3Fe4F6";
	str >> CXSC_Catalan[14];
	str = "+1A86EB34EAD01Ae4C0";
	str >> CXSC_Catalan[15];
	str = "+1C68C37800513Be485";
	str >> CXSC_Catalan[16];
	str = "+1D46EBB334D7C9e44E";
	str >> CXSC_Catalan[17];
	str = "-1944C5E2711625e417";
	str >> CXSC_Catalan[18];
	str = "-17885C649BB92Fe3E1";
	str >> CXSC_Catalan[19];
	str = "+1A2A0CEE24DD91e3A9";
	str >> CXSC_Catalan[20];
	str = "+159AEC52EB2869e372";
	str >> CXSC_Catalan[21];
	str = "-1D26976389F1E1e339";
	str >> CXSC_Catalan[22];
	str = "+1E9AF9FF2E2FB1e302";
	str >> CXSC_Catalan[23];
	str = "+1E8B66677323FEe2CC";
	str >> CXSC_Catalan[24];
	str = "+164BBD8A306F6Ae296";
	str >> CXSC_Catalan[25];
	str = "-1EE36A15C4872Be25F";
	str >> CXSC_Catalan[26];
	str = "+1C3A35B39DC2FFe228";
	str >> CXSC_Catalan[27];
	str = "-1CCAF1572CDFC2e1F1";
	str >> CXSC_Catalan[28];
	str = "+15FB135902BFEEe1BA";
	str >> CXSC_Catalan[29];
	str = "+19FDE4873721BAe183";
	str >> CXSC_Catalan[30];
	str = "-17ABB7B5115456e14D";
	str >> CXSC_Catalan[31];
	str = "+1458F7F79FA825e117";
	str >> CXSC_Catalan[32];
	str = "-1416ED1E24CEFDe0E1";
	str >> CXSC_Catalan[33];
	str = "+15A6293C127A02e0A9";
	str >> CXSC_Catalan[34];
	str = "-1F1AABC6E5593Ce073";
	str >> CXSC_Catalan[35];
	str = "+1A7A8FB50B3479e03D";
	str >> CXSC_Catalan[36];
	str = "+1B853813268EF2e005";
	str >> CXSC_Catalan[37];
	str = "+10000000000003e000";
	str >> CXSC_Catalan[38];
	str = "+10000000000004e000";
	str >> CXSC_Catalan[39];

	CXSC_Catalan_initialized = true;
	std::cout << RestoreOpt;
    } 
    stagprec = stagmax;
    y = adjust(l_interval(0));

    for (int i=0; i<=stagmax; i++)
	y[i+1] = CXSC_Catalan[i];

    stagprec = stagsave;
    y = adjust(y);

    return lx_interval(-1022,y);
} // Catalan

static real CXSC_sqrt5[40]; // CXSC_sqrt5[0],...,CXSC_sqrt5[39]
static bool CXSC_sqrt5_initialized = false;

lx_interval sqrt5_lx_interval() throw()
// Inclusion of sqrt(5), Blomquist, 15.06.2007;
{
    l_interval y;
    int stagsave = stagprec,
	stagmax = 39;
    if (!CXSC_sqrt5_initialized)
    {
	std::string str;
	std::cout << SaveOpt;
	std::cout << Hex;
	str = "+11E3779B97F4A8e7FD";
	str >> CXSC_sqrt5[0];
	str = "-1F506319FCFD19e7C6";
	str >> CXSC_sqrt5[1];
	str = "+1B906821044ED8e790";
	str >> CXSC_sqrt5[2];
	str = "-18BB1B5C0F272Ce758";
	str >> CXSC_sqrt5[3];
	str = "+11D0C18E952768e721";
	str >> CXSC_sqrt5[4];
	str = "-1E9D585B0901F9e6E8";
	str >> CXSC_sqrt5[5];
	str = "-1C7DD252073EC0e6B2";
	str >> CXSC_sqrt5[6];
	str = "-1FCEF21EDAF7FAe67C";
	str >> CXSC_sqrt5[7];
	str = "+160EB25D20799Be63E";
	str >> CXSC_sqrt5[8];
	str = "-1C90F95285168Fe605";
	str >> CXSC_sqrt5[9];
	str = "+1E1DFA160E75BCe5CF";
	str >> CXSC_sqrt5[10];
	str = "-10A08E66CB368Ce593";
	str >> CXSC_sqrt5[11];
	str = "+1C5371682CADD1e55D";
	str >> CXSC_sqrt5[12];
	str = "-1998100220F4EDe526";
	str >> CXSC_sqrt5[13];
	str = "+1C6771A0968663e4F0";
	str >> CXSC_sqrt5[14];
	str = "+1DFB9E3C86CA7Ce4BA";
	str >> CXSC_sqrt5[15];
	str = "-18AE38ED5304B1e483";
	str >> CXSC_sqrt5[16];
	str = "+182A5FEC507706e44D";
	str >> CXSC_sqrt5[17];
	str = "-1B5191A18C5647e415";
	str >> CXSC_sqrt5[18];
	str = "+1F3AA4DB287AE4e3DD";
	str >> CXSC_sqrt5[19];
	str = "+13CCCBA48E9CF3e3A7";
	str >> CXSC_sqrt5[20];
	str = "-1BA6DC6F5C5A29e36F";
	str >> CXSC_sqrt5[21];
	str = "-1C87E14FE4B628e339";
	str >> CXSC_sqrt5[22];
	str = "+19D3E854210678e303";
	str >> CXSC_sqrt5[23];
	str = "+1D54807166A5B3e2CC";
	str >> CXSC_sqrt5[24];
	str = "+1D6987A242DB8De296";
	str >> CXSC_sqrt5[25];
	str = "+11DEC88A541BA8e260";
	str >> CXSC_sqrt5[26];
	str = "+1B91971F345A99e228";
	str >> CXSC_sqrt5[27];
	str = "-1758726EB11AAFe1F2";
	str >> CXSC_sqrt5[28];
	str = "+115613474B3372e1BA";
	str >> CXSC_sqrt5[29];
	str = "+150CCED228BEBFe184";
	str >> CXSC_sqrt5[30];
	str = "-13BFAB17A99623e14E";
	str >> CXSC_sqrt5[31];
	str = "-17397D0B07EFCEe115";
	str >> CXSC_sqrt5[32];
	str = "+157D728069CE1Ee0DC";
	str >> CXSC_sqrt5[33];
	str = "+10B046030D58E1e0A6";
	str >> CXSC_sqrt5[34];
	str = "-1A8870A118821Ce06E";
	str >> CXSC_sqrt5[35];
	str = "-1BD96261F224FBe038";
	str >> CXSC_sqrt5[36];
	str = "+1FDBEC758D20C8e002";
	str >> CXSC_sqrt5[37];
	str = "+10000000000000e000";
	str >> CXSC_sqrt5[38];
	str = "+10000000000001e000";
	str >> CXSC_sqrt5[39];

	CXSC_sqrt5_initialized = true;
	std::cout << RestoreOpt;
    } 
    stagprec = stagmax;
    y = adjust(l_interval(0));

    for (int i=0; i<=stagmax; i++)
	y[i+1] = CXSC_sqrt5[i];

    stagprec = stagsave;
    y = adjust(y);

    return lx_interval(-1021,y);
} // sqrt(5)

static real CXSC_sqrt7[40]; // CXSC_sqrt7[0],...,CXSC_sqrt7[39]
static bool CXSC_sqrt7_initialized = false;

lx_interval sqrt7_lx_interval() throw()
// Inclusion of sqrt(7), Blomquist, 15.06.2007;
{
    l_interval y;
    int stagsave = stagprec,
	stagmax = 39;
    if (!CXSC_sqrt7_initialized)
    {
	std::string str;
	std::cout << SaveOpt;
	std::cout << Hex;
	str = "+152A7FA9D2F8EAe7FD";
	str >> CXSC_sqrt7[0];
	str = "-121C62B033C079e7C7";
	str >> CXSC_sqrt7[1];
	str = "-177CAAD6200612e78E";
	str >> CXSC_sqrt7[2];
	str = "-1EFA880DC72D64e756";
	str >> CXSC_sqrt7[3];
	str = "-171D206D5B1A4Ce71C";
	str >> CXSC_sqrt7[4];
	str = "+119392FA9B0494e6E3";
	str >> CXSC_sqrt7[5];
	str = " +17BB8A64890057e6AA";
	str >> CXSC_sqrt7[6];
	str = "-17E89300383DDEe674";
	str >> CXSC_sqrt7[7];
	str = "+130FB7AF68A6FBe63E";
	str >> CXSC_sqrt7[8];
	str = "+1322281D303D36e606";
	str >> CXSC_sqrt7[9];
	str = "+1996109A16D3B1e5D0";
	str >> CXSC_sqrt7[10];
	str = "+1F239C301DFBB4e599";
	str >> CXSC_sqrt7[11];
	str = "-1B5CA40AB771A2e560";
	str >> CXSC_sqrt7[12];
	str = "-1675711487FEAAe527";
	str >> CXSC_sqrt7[13];
	str = "+122CB7FA26ABA5e4F1";
	str >> CXSC_sqrt7[14];
	str = "+1059211B7D5398e4BA";
	str >> CXSC_sqrt7[15];
	str = "-10F15BFA46EB7Fe484";
	str >> CXSC_sqrt7[16];
	str = "+15AB71566CE72Be44E";
	str >> CXSC_sqrt7[17];
	str = "-1386BDCA3845C7e417";
	str >> CXSC_sqrt7[18];
	str = "+158978E1A883F0e3E1";
	str >> CXSC_sqrt7[19];
	str = "+1F8A772604CAF1e3AB";
	str >> CXSC_sqrt7[20];
	str = "-169730DF195425e374";
	str >> CXSC_sqrt7[21];
	str = "+121341C1C58312e33D";
	str >> CXSC_sqrt7[22];
	str = "+1117A6A8D5A423e307";
	str >> CXSC_sqrt7[23];
	str = "-12D6F266C4AE34e2D0";
	str >> CXSC_sqrt7[24];
	str = "-1B8D3340BC4497e299";
	str >> CXSC_sqrt7[25];
	str = "-1E0712BA5C43CFe263";
	str >> CXSC_sqrt7[26];
	str = "+14BF59F22FDCEDe22D";
	str >> CXSC_sqrt7[27];
	str = "-189028E9147846e1F3";
	str >> CXSC_sqrt7[28];
	str = "-1BBE8044991D23e1BB";
	str >> CXSC_sqrt7[29];
	str = "+1B07A12AB80C0Ae185";
	str >> CXSC_sqrt7[30];
	str = "+1970A29C5148A7e14E";
	str >> CXSC_sqrt7[31];
	str = "-18DBF939495F77e117";
	str >> CXSC_sqrt7[32];
	str = "-1865ECB3F101B4e0E1";
	str >> CXSC_sqrt7[33];
	str = "-1DCDFCFE6050DFe0A4";
	str >> CXSC_sqrt7[34];
	str = "+1E635021BBF848e06B";
	str >> CXSC_sqrt7[35];
	str = "+15C53CF344AF94e035";
	str >> CXSC_sqrt7[36];
	str = "+124CB182B20E69e000";
	str >> CXSC_sqrt7[37];
	str = "-10000000000001e000";
	str >> CXSC_sqrt7[38];
	str = "-10000000000000e000";
	str >> CXSC_sqrt7[39];

	CXSC_sqrt7_initialized = true;
	std::cout << RestoreOpt;
    } 
    stagprec = stagmax;
    y = adjust(l_interval(0));

    for (int i=0; i<=stagmax; i++)
	y[i+1] = CXSC_sqrt7[i];

    stagprec = stagsave;
    y = adjust(y);

    return lx_interval(-1021,y);
} // sqrt(7)

static real CXSC_One_m[3]; // CXSC_One_m[0], ... ,CXSC_One_m[2]
static bool CXSC_One_m_initialized = false;

lx_interval One_m_lx_interval() throw()
// Inclusion of 2^(-1023)*(2^(+1023) - 2^(-1074)) = 1-2^(-2097), 
// Blomquist, 26.07.2007;
{
    l_interval y;
    int stagsave = stagprec,
	stagmax = 2;

    if (!CXSC_One_m_initialized)
    {
	std::string str;
	std::cout << SaveOpt;
	std::cout << Hex;
	str = "+10000000000000e7FE";
	str >> CXSC_One_m[0];
	str = "-10000000000001e000";
	str >> CXSC_One_m[1];
	str = "-10000000000001e000";
	str >> CXSC_One_m[2];
	CXSC_One_m_initialized = true;
	std::cout << RestoreOpt;
    } 
    stagprec = stagmax;
    y = adjust(l_interval(0));

    for (int i=0; i<=stagmax; i++)
	y[i+1] = CXSC_One_m[i];

    stagprec = stagsave;
    y = adjust(y);

    return lx_interval(-1023,y);
} // One_m_lx_interval()

static real CXSC_One_p[3]; // CXSC_One_p[0], ... ,CXSC_One_p[2]
static bool CXSC_One_p_initialized = false;

lx_interval One_p_lx_interval() throw()
// Inclusion of 2^(-1023)*(2^(+1023) + 2^(-1074)) = 1+2^(-2097), 
// Blomquist, 27.07.2007;
{
    l_interval y;
    int stagsave = stagprec,
	stagmax = 2;

    if (!CXSC_One_p_initialized)
    {
	std::string str;
	std::cout << SaveOpt;
	std::cout << Hex;
	str = "+10000000000000e7FE";
	str >> CXSC_One_p[0];
	str = "+10000000000001e000";
	str >> CXSC_One_p[1];
	str = "+10000000000001e000";
	str >> CXSC_One_p[2];
	CXSC_One_p_initialized = true;
	std::cout << RestoreOpt;
    }
    stagprec = stagmax;
    y = adjust(l_interval(0));

    for (int i=0; i<=stagmax; i++)
	y[i+1] = CXSC_One_p[i];

    stagprec = stagsave;
    y = adjust(y);

    return lx_interval(-1023,y);
} // One_p_lx_interval()


} // namespace cxsc
